@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2004-06-06
+ *  Updated   : 2005-03-12
  *  Notes     :
  *
- * Copyright (c) 1997-2004 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,6 +45,9 @@
 #ifndef RFC2046_H
 #	include <rfc2046.h>
 #endif /* !RFC2046_H */
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
 
 /*
  * The prototypes bracketed by DECL_xxxx ifdef's are used to get moderately
@@ -325,7 +328,6 @@ extern char **news_headers_to_display_array;
 extern char **news_headers_to_not_display_array;
 extern char *OPT_CHAR_list[];
 extern char *OPT_STRING_list[];
-extern char *ch_post_process;
 extern char *nntp_server;
 extern char active_times_file[PATH_LEN];
 extern char bug_addr[LEN];
@@ -473,9 +475,9 @@ extern constext txt_cannot_create[];
 extern constext txt_cannot_create_uniq_name[];
 extern constext txt_cannot_find_base_art[];
 extern constext txt_cannot_get_nntp_server_name[];
-#if defined(M_UNIX) && !defined(USE_TERMINFO)
+#if !defined(USE_CURSES) && defined(M_UNIX) && !defined(USE_TERMINFO)
 	extern constext txt_cannot_get_term_entry[];
-#endif /* M_UNIX && !USE_TERMINFO */
+#endif /* !USE_CURSES && M_UNIX && !USE_TERMINFO */
 extern constext txt_cannot_open[];
 extern constext txt_cannot_open_for_saving[];
 extern constext txt_cannot_post[];
@@ -716,6 +718,8 @@ extern constext txt_help_global_quit_tin[];
 extern constext txt_help_global_redraw_screen[];
 extern constext txt_help_global_save[];
 extern constext txt_help_global_auto_save[];
+extern constext txt_help_global_scroll_down[];
+extern constext txt_help_global_scroll_up[];
 extern constext txt_help_global_search_auth_backwards[];
 extern constext txt_help_global_search_auth_forwards[];
 extern constext txt_help_global_search_body[];
@@ -825,7 +829,10 @@ extern constext txt_keymap_invalid_name[];
 #ifdef DEBUG
 	extern constext txt_keymap_redef[];
 #endif /* DEBUG */
+#if 0 /* ununsed */
 extern constext txt_keymap_conflict[];
+#endif /* 0 */
+extern constext txt_keymap_upgraded[];
 extern constext txt_kill_from[];
 extern constext txt_kill_lines[];
 extern constext txt_kill_menu[];
@@ -901,11 +908,13 @@ extern constext txt_no_responses[];
 extern constext txt_no_resps_in_thread[];
 extern constext txt_no_search_string[];
 extern constext txt_no_subject[];
-extern constext txt_no_term_clear_eol[];
-extern constext txt_no_term_clear_eos[];
-extern constext txt_no_term_clearscreen[];
-extern constext txt_no_term_cursor_motion[];
-extern constext txt_no_term_set[];
+#ifndef USE_CURSES
+	extern constext txt_no_term_clear_eol[];
+	extern constext txt_no_term_clear_eos[];
+	extern constext txt_no_term_clearscreen[];
+	extern constext txt_no_term_cursor_motion[];
+	extern constext txt_no_term_set[];
+#endif /* !USE_CURSES */
 extern constext txt_no_viewer_found[];
 extern constext txt_no_xover_support[];
 extern constext txt_not_exist[];
@@ -972,6 +981,9 @@ extern constext txt_reading_newsrc[];
 #ifdef NNTP_ABLE
 	extern constext txt_reconnect_to_news_server[];
 #endif /* NNTP_ABLE */
+#ifdef HAVE_GETTIMEOFDAY
+	extern constext txt_remaining[];
+#endif /* HAVE_GETTIMEOFDAY */
 extern constext txt_remove_bogus[];
 extern constext txt_rename_error[];
 extern constext txt_reply_to_author[];
@@ -996,7 +1008,9 @@ extern constext txt_saved_to[];
 extern constext txt_saved_to_range[];
 extern constext txt_saving[];
 extern constext txt_screen_init_failed[];
-extern constext txt_screen_too_small[];
+#ifndef USE_CURSES
+	extern constext txt_screen_too_small[];
+#endif /* !USE_CURSES */
 extern constext txt_screen_too_small_exiting[];
 extern constext txt_search_backwards[];
 extern constext txt_search_body[];
@@ -1038,6 +1052,7 @@ extern constext txt_supersede_group[];
 extern constext txt_superseding_art[];
 extern constext txt_suspended_message[];
 extern constext txt_tab[];
+extern constext txt_tex[];
 extern constext txt_there_is_no_news[];
 extern constext txt_thread_upper[];
 extern constext txt_thread_com[];
@@ -1198,7 +1213,6 @@ extern int cCOLS;
 extern int cLINES;
 extern int curr_line;
 extern int debug;
-extern int i_key_search_last;
 extern int input_context;
 extern int iso2asc_supported;
 extern int last_resp;
@@ -1287,6 +1301,8 @@ extern t_bool batch_mode;
 extern t_bool verbose;
 extern t_bool xref_supported;
 
+extern t_function last_search;
+
 extern t_menu selmenu;
 extern t_menu grpmenu;
 extern t_menu *currmenu;
@@ -1490,7 +1506,9 @@ extern struct opttxt txt_mono_markstar;
 extern struct opttxt txt_mono_markdash;
 extern struct opttxt txt_mono_markslash;
 extern struct opttxt txt_mono_markstroke;
-extern struct opttxt txt_mm_charset;
+#ifndef CHARSET_CONVERSION
+	extern struct opttxt txt_mm_charset;
+#endif /* CHARSET_CONVERSION */
 extern struct opttxt txt_metamail_prog;
 extern struct opttxt txt_news_headers_to_display;
 extern struct opttxt txt_news_headers_to_not_display;
