@@ -6,7 +6,7 @@
  *  Updated   : 2004-01-02
  *  Notes     : #defines and static data for policy configuration
  *
- * Copyright (c) 1999-2004 Ralf Doeblitz <doeblitz@gmx.de>
+ * Copyright (c) 1999-2005 Ralf Doeblitz <doeblitz@gmx.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -205,6 +205,7 @@
  * .dd  former German Democratic Republic, now .de
  * .fx  France, Metropolitan
  * .yd  Democratic Yemen, now .ye
+ * .wg  West Bank and Gaza, now .ps
  * .zr  former Zaire, now .cd
  *
  * user-assigned code elements:
@@ -220,11 +221,11 @@
  * .cs  Serbia and Montenegro
  * .eu  European Union (planned for 2004-11)
  *
- * 'expireing' TLDs:
+ * xccTLDs:
  * .gb  United Kingdom, use .uk
- *(.hk  Hong Kong, now .cn)
+ * .hk  Hong Kong, now .cn
  * .su  former USSR, now .ru
- *(.yu  former Yugoslavia)
+ * .yu  former Yugoslavia
  */
 
 static char gnksa_country_codes[26*26] = {
@@ -278,7 +279,11 @@ static const char *gnksa_domain_list[] = {
 	"info",	/* Unrestricted use */
 	"name",	/* For registration by individuals */
 	"museum",	/* Museums */
-	/* 3 more gTLDs to come in 2004 (.travel is likely to be one of them) */
+	/*
+	 * more gTLDs to come in 2004, proposed are:
+	 *  .asia   .cat    .jobs   .mail   .mobi
+	 *  .post   .tel    .travel .xxx
+	 */
 #	if 0		/* $DEAD */
 	"uucp",
 	"bitnet",
