@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2004-06-12
+ *  Updated   : 2005-03-07
  *  Notes     :
  *
- * Copyright (c) 1997-2004 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2005 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,10 @@
 #ifndef PROTO_H
 #	define PROTO_H 1
 
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
+
 /* This fixes ambiguities on platforms that don't distinguish extern case */
 #ifdef CASE_PROBLEM
 #	define Raw tin_raw
@@ -78,8 +82,8 @@ extern void write_attributes_file(const char *file);
 
 /* charset.c */
 extern char *convert_to_printable(char *buf);
+extern char *convert_body2printable(char* buf);
 extern t_bool is_art_tex_encoded(FILE *fp);
-extern void convert_body2printable(char* buf);
 extern void convert_iso2asc(char *iso, char **asc_buffer, int *max_line_len, int t);
 extern void convert_tex2iso(char *from, char *to);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -87,19 +91,16 @@ extern void convert_tex2iso(char *from, char *to);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* color.c */
+extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
 #ifdef HAVE_COLOR
 	extern void bcol(int color);
 	extern void fcol(int color);
 #endif /* HAVE_COLOR */
-extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
 
 /* config.c */
 extern char **ulBuildArgv(char *cmd, int *new_argc);
-extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, int option);
 extern char *quote_space_to_dash(char *str);
 extern const char *print_boolean(t_bool value);
-extern int change_config_file(struct t_group *group);
-extern int option_row(int option);
 extern t_bool match_boolean(char *line, const char *pat, t_bool *dst);
 extern t_bool match_integer(char *line, const char *pat, int *dst, int maxval);
 extern t_bool match_list(char *line, constext *pat, constext *const *table, size_t tablelen, int *dst);
@@ -108,8 +109,6 @@ extern t_bool match_string(char *line, const char *pat, char *dst, size_t dstlen
 extern t_bool read_config_file(char *file, t_bool global_file);
 extern void quote_dash_to_space(char *str);
 extern void read_server_config(void);
-extern void refresh_config_page(int act_option);
-extern void show_menu_help(const char *help_message);
 extern void write_config_file(char *file);
 
 /* cook.c */
@@ -122,22 +121,11 @@ extern OUTC_RETTYPE outchar(OUTC_ARGS);
 extern int InitScreen(void);
 extern int RawState(void);
 extern int ReadCh(void);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && !defined(USE_CURSES)
-	extern wint_t ReadWch(void);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE && !USE_CURSES */
 extern int get_arrow_key(int prech);
 extern int get_termcaps(void);
-extern void ClearScreen(void);
-extern void CleartoEOLN(void);
-extern void CleartoEOS(void);
-extern void ScrollScreen(int lines_to_scroll);
-extern void SetScrollRegion(int topline, int bottomline);
 extern void EndInverse(void);
 extern void EndWin(void);
 extern void InitWin(void);
-#ifndef USE_CURSES
-	extern void MoveCursor(int row, int col);
-#endif /* USE_CURSES */
 extern void Raw(int state);
 extern void StartInverse(void);
 extern void cursoroff(void);
@@ -149,6 +137,17 @@ extern void set_xclick_off(void);
 extern void set_xclick_on(void);
 extern void setup_screen(void);
 extern void word_highlight_string(int row, int col, int size, int color);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && !defined(USE_CURSES)
+	extern wint_t ReadWch(void);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE && !USE_CURSES */
+#ifndef USE_CURSES
+	extern void ClearScreen(void);
+	extern void CleartoEOLN(void);
+	extern void CleartoEOS(void);
+	extern void MoveCursor(int row, int col);
+	extern void ScrollScreen(int lines_to_scroll);
+	extern void SetScrollRegion(int topline, int bottomline);
+#endif /* USE_CURSES */
 #if 0
 	extern void ToggleInverse(void);
 #endif /* 0 */
@@ -156,7 +155,7 @@ extern void word_highlight_string(int row, int col, int size, int color);
 /* debug.c */
 #ifdef DEBUG
 	extern void debug_delete_files(void);
-	extern void debug_nntp(const char *func, const char *line);
+	extern void debug_nntp(const char *func, const char *fmt, ...);
 	extern void debug_print_active(void);
 	extern void debug_print_arts(void);
 	extern void debug_print_filters(void);
@@ -202,13 +201,15 @@ extern void top_of_list(void);
 extern void end_of_list(void);
 extern void move_to_item(int n);
 extern void prompt_item_num(int ch, const char *prompt);
+extern void scroll_down(void);
+extern void scroll_up(void);
 extern void set_first_screen_item(void);
 
 /* group.c */
 extern int find_new_pos(int old_top, long old_artnum, int cur_pos);
 extern int group_page(struct t_group *group);
 extern void clear_note_area(void);
-extern void mark_screen(int level, int screen_row, int screen_col, const char *value);
+extern void mark_screen(int screen_row, int screen_col, const char *value);
 extern void pos_first_unread_thread(void);
 extern void show_group_page(void);
 
@@ -248,7 +249,6 @@ extern void joinpath(char *result, const char *dir, const char *file);
 /* keymap.c */
 extern char *printascii(char *buf, int ch);
 extern t_bool read_keymap_file(void);
-extern void build_keymaps(void);
 extern void free_keymaps(void);
 
 /* langinfo.c */
@@ -287,10 +287,6 @@ extern void grp_del_mail_art(struct t_article *article);
 	extern void write_mail_active_file(void);
 #endif /* HAVE_MH_MAIL_HANDLING */
 
-/* mimetypes.c */
-extern void lookup_mimetype(const char *ext, t_part *part);
-extern t_bool lookup_extension(char *extension, size_t ext_len, const char *major, const char *minor);
-
 /* main.c */
 extern int main(int argc, char *argv[]);
 extern int read_cmd_line_groups(void);
@@ -314,6 +310,10 @@ extern void *my_realloc1(const char *file, int line, void *p, size_t size);
 	extern void init_screen_array(t_bool allocate);
 #endif /* !USE_CURSES */
 
+/* mimetypes.c */
+extern void lookup_mimetype(const char *ext, t_part *part);
+extern t_bool lookup_extension(char *extension, size_t ext_len, const char *major, const char *minor);
+
 /* misc.c */
 extern char *buffer_to_ascii(char *c);
 extern char *escape_shell_meta(const char *source, int quote_area);
@@ -336,6 +336,7 @@ extern int parse_from(const char *from, char *address, char *realname);
 extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
 extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group);
 extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, char *format);
+extern int tin_version_info(FILE *fp);
 extern long file_mtime(const char *file);
 extern long file_size(const char *file);
 extern t_bool backup_file(const char *filename, const char *backupname);
@@ -350,6 +351,7 @@ extern void cleanup_tmp_files(void);
 extern void copy_body(FILE *fp_ip, FILE *fp_op, char *prefix, char *initl, t_bool raw_data);
 extern void create_index_lock_file(char *the_lock_file);
 extern void dir_name(const char *fullpath, char *dir);
+extern void draw_mark_selected(int i);
 extern void draw_percent_mark(long cur_num, long max_num);
 extern void get_author(t_bool thread, struct t_article *art, char *str, size_t len);
 extern void get_cwd(char *buf);
@@ -362,9 +364,9 @@ extern void show_inverse_video_status(void);
 extern void strip_name(const char *from, char *address);
 extern void tin_done(int ret);
 extern void toggle_inverse_video(void);
-#if defined(CHARSET_CONVERSION) || defined(HAVE_UNICODE_NORMALIZATION)
+#if defined(CHARSET_CONVERSION) || (defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE))
 	extern char *utf8_valid(char *line);
-#endif /* CHARSET_CONVERSION || HAVE_UNICODE_NORMALIZATION */
+#endif /* CHARSET_CONVERSION || (MULTIBYTE_ABLE && !NO_LOCALE) */
 #ifdef CHARSET_CONVERSION
 	extern t_bool buffer_to_network(char *line, int mmnwcharset);
 #endif /* CHARSET_CONVERSION */
@@ -406,9 +408,6 @@ extern void set_default_bitmap(struct t_group *group);
 #endif /* DEBUG_NEWSRC */
 
 /* nntplib.c */
-#ifdef NNTP_ABLE
-	extern FILE *nntp_command(const char *, int, char *, size_t);
-#endif /* NNTP_ABLE */
 extern FILE *get_nntp_fp(FILE *fp);
 extern FILE *get_nntp_wr_fp(FILE *fp);
 extern char *getserverbyfile(const char *file);
@@ -419,11 +418,23 @@ extern int nntp_open(void);
 extern void nntp_close(void);
 extern void put_server(const char *string);
 extern void u_put_server(const char *string);
+#ifdef NNTP_ABLE
+	extern FILE *nntp_command(const char *, int, char *, size_t);
+#endif /* NNTP_ABLE */
 
 /* nrctbl.c */
 extern int get_newsrcname(char *newsrc_name, const char *nntpserver_name);
 extern void get_nntpserver(char *nntpserver_name, char *nick_name);
 
+/* options_menu.c */
+extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, int option);
+extern int change_config_file(struct t_group *group);
+extern int option_row(int option);
+extern t_bool option_is_visible(int option);
+extern void check_score_defaults(void);
+extern void refresh_config_page(int act_option);
+extern void show_menu_help(const char *help_message);
+
 /* page.c */
 extern int show_page(struct t_group *group, int respnum, int *threadnum);
 extern void display_info_page(int part);
@@ -467,7 +478,6 @@ extern t_bool user_posted_messages(void);
 extern void init_postinfo(void);
 extern void quick_post_article(t_bool postponed_only);
 #ifdef USE_CANLOCK
-	extern const char *build_cankey(const char *messageid, const char *secret);
 	extern const char *build_canlock(const char *messageid, const char *secret);
 	extern char *get_secret(void);
 #endif /* USE_CANLOCK */
@@ -476,7 +486,7 @@ extern void quick_post_article(t_bool postponed_only);
 extern char *prompt_string_default(const char *prompt, char *def, const char *failtext, int history);
 extern char *sized_message(char **result, const char *format, const char *subject);
 extern int prompt_num(int ch, const char *prompt);
-extern int prompt_yn(int line, const char *prompt, t_bool default_answer);
+extern int prompt_yn(const char *prompt, t_bool default_answer);
 extern int prompt_msgid(void);
 extern t_bool prompt_default_string(const char *prompt, char *buf, int buf_len, char *default_prompt, int which_hist);
 extern t_bool prompt_menu_string(int line, const char *prompt, char *var);
@@ -514,6 +524,8 @@ extern t_mailcap *get_mailcap_entry(t_part *part, const char *path);
 extern void free_mailcap(t_mailcap *tmailcap);
 
 /* rfc2045.c */
+extern int read_decoded_base64_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read, char **rest);
+extern int read_decoded_qp_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read);
 extern void rfc1521_encode(char *line, FILE *f, int e);
 
 /* rfc2046.c */
@@ -542,12 +554,13 @@ extern void compose_mail_text_plain(const char *filename, struct t_group *group)
 extern int check_start_save_any_news(int function, t_bool catchup);
 extern t_bool create_path(const char *path);
 extern t_bool expand_save_filename(char *outpath, const char *path);
-extern t_bool post_process_files(int proc_type_ch, t_bool auto_delete);
+extern t_bool post_process_files(t_function proc_type_type, t_bool auto_delete);
 extern t_bool save_and_process_art(t_openartinfo *artinfo, struct t_article *artptr, t_bool is_mailbox, const char *inpath, int max, t_bool post_process);
 extern void decode_save_mime(t_openartinfo *art, t_bool postproc);
 extern void print_art_seperator_line(FILE *fp, t_bool is_mailbox);
 
 /* screen.c */
+extern char *fmt_message(const char *fmt, va_list ap);
 extern void center_line(int line, t_bool inverse, const char *str);
 extern void clear_message(void);
 extern void draw_arrow_mark(int line);
@@ -564,17 +577,17 @@ extern void wait_message(unsigned int sdelay, const char *fmt, ...);
 
 /* search.c */
 extern int get_search_vectors(int *start, int *end);
-extern int search(int key, int current_art, t_bool forward, t_bool repeat);
+extern int search(t_function func, int current_art, t_bool repeat);
 extern int search_active(t_bool forward, t_bool repeat);
 extern int search_article(t_bool forward, t_bool repeat, int start_line, int lines, t_lineinfo *line, int reveal_ctrl_l_lines, FILE *fp);
 extern int search_config(t_bool forward, t_bool repeat, int current, int last);
 extern int search_body(struct t_group *group, int current_art, t_bool repeat);
+extern void reset_srch_offsets(void);
 
 /* select.c */
 extern int add_my_group(const char *group, t_bool add);
 extern int choose_new_group(void);
 extern int skip_newgroups(void);
-extern void draw_group_arrow(void);
 extern void selection_page(int start_groupnum, int num_cmd_line_groups);
 extern void show_selection_page(void);
 extern void toggle_my_groups(const char *group);
@@ -583,7 +596,7 @@ extern void toggle_my_groups(const char *group);
 extern void msg_write_signature(FILE *fp, t_bool flag, struct t_group *thisgroup);
 
 /* signal.c */
-extern RETSIGTYPE(*sigdisp (int sig, RETSIGTYPE (*func)(SIG_ARGS))) (SIG_ARGS);
+extern RETSIGTYPE(*sigdisp (int signum, RETSIGTYPE (*func)(SIG_ARGS))) (SIG_ARGS);
 extern t_bool set_win_size(int *num_lines, int *num_cols);
 extern void allow_resize(t_bool allow);
 extern void handle_resize(t_bool repaint);
@@ -599,15 +612,16 @@ extern char *eat_tab(char *s);
 extern char *fmt_string(const char *fmt, ...);
 extern char *my_strdup(const char *str);
 extern char *str_trim(char *string);
+extern char *strunc(const char *message, int len);
 extern char *tin_ltoa(long value, int digits);
 extern char *tin_strtok(char *str, const char *delim);
-#if !defined(HAVE_STRCASESTR) || defined(DECL_STRCASESTR)
-	extern const char *strcasestr(const char *haystack, const char *needle);
-#endif /* !HAVE_STRCASESTR || DECL_STRCASESTR */
 extern int sh_format(char *dst, size_t len, const char *fmt, ...);
 extern size_t mystrcat(char **t, const char *s);
 extern void my_strncpy(char *p, const char *q, size_t n);
 extern void str_lwr(char *str);
+#if !defined(HAVE_STRCASESTR) || defined(DECL_STRCASESTR)
+	extern const char *strcasestr(const char *haystack, const char *needle);
+#endif /* !HAVE_STRCASESTR || DECL_STRCASESTR */
 #ifndef HAVE_STRPBRK
 	extern char *strpbrk(char *str1, char *str2);
 #endif /* !HAVE_STRPBRK */
@@ -639,12 +653,11 @@ extern void str_lwr(char *str);
 #endif /* !HAVE_STRRSTR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern char *wchar_t2char(const wchar_t *wstr);
-	extern void wcspart(wchar_t *to, const wchar_t *from, int columns, int size_to, t_bool pad);
 	extern wchar_t *char2wchar_t(const char *str);
-	extern wchar_t *wstrunc(const wchar_t *wmessage, wchar_t *wbuf, size_t wbuf_len, int len);
+	extern wchar_t *wcspart(const wchar_t *wstr, int columns, t_bool pad);
+	extern wchar_t *wstrunc(const wchar_t *wmessage, int len);
 	extern wchar_t *my_wcsdup(const wchar_t *wstr);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-extern char *strunc(const char *message, char *buf, size_t buf_len, int len);
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	extern char *render_bidi(const char *str, t_bool *is_rtl);
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
@@ -688,7 +701,6 @@ extern int stat_thread(int n, struct t_art_stat *sbuf);
 extern int which_response(int n);
 extern int which_thread(int n);
 extern int thread_page(struct t_group *group, int respnum, int thread_depth, t_pagerinfo *page);
-extern void draw_line(int i, int magic);
 extern void fixup_thread(int respnum, t_bool redraw);
 
 /* version.c */
