@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2004-06-06
+ *  Updated   : 2005-02-15
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,15 +41,9 @@
 #ifndef VERSION_H
 #	include "version.h"
 #endif /* !VERSION_H */
-#ifndef TINTBL_H
-#	include "tincfg.h"
-#endif /* !TINTBL_H */
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
 #ifndef TNNTP_H
 #	include "tnntp.h"
 #endif /* TNNTP_H */
@@ -57,26 +51,12 @@
 /*
  * local prototypes
  */
-static int get_option_num(int act_option);
-static int set_option_num(int option);
-static t_bool OptionOnPage(int option);
 static t_bool match_item(char *line, const char *pat, char *dst, size_t dstlen);
 static t_bool rc_update(FILE *fp);
-static void RepaintOption(int option);
-static void check_score_defaults(void);
-static void highlight_option(int option);
-static void print_any_option(int act_option);
-static void print_option(enum option_enum the_option);
-static void redraw_screen(int option);
-static void show_config_page(void);
-static void unhighlight_option(int option);
 static void write_server_config(void);
 #ifdef HAVE_COLOR
 	static t_bool match_color(char *line, const char *pat, int *dst, int max);
 #endif /* HAVE_COLOR */
-#ifdef USE_CURSES
-	static void DoScroll(int jump);
-#endif /* USE_CURSES */
 
 
 #define DASH_TO_SPACE(mark)	((char) (mark == '_' ? ' ' : mark))
@@ -1227,40 +1207,46 @@ write_config_file(
 	fprintf(fp, "col_urls=%d\n\n", tinrc.col_urls);
 #endif /* HAVE_COLOR */
 
-#ifdef XFACE_ABLE
-	fprintf(fp, _(txt_use_slrnface.tinrc));
-	fprintf(fp, "use_slrnface=%s\n\n", print_boolean(tinrc.use_slrnface));
-#endif /*XFACE_ABLE */
-
 	fprintf(fp, _(txt_url_highlight.tinrc));
 	fprintf(fp, "url_highlight=%s\n\n", print_boolean(tinrc.url_highlight));
 
 	fprintf(fp, _(txt_word_highlight.tinrc));
 	fprintf(fp, "word_highlight=%s\n\n", print_boolean(tinrc.word_highlight));
 
-	fprintf(fp, _(txt_wrap_column.tinrc));
-	fprintf(fp, "wrap_column=%d\n\n", tinrc.wrap_column);
-
 	fprintf(fp, _(txt_word_h_display_marks.tinrc));
 	fprintf(fp, "word_h_display_marks=%d\n\n", tinrc.word_h_display_marks);
 
 #ifdef HAVE_COLOR
 	fprintf(fp, _(txt_col_markstar.tinrc));
-	fprintf(fp, "col_markstar=%d\n", tinrc.col_markstar);
-	fprintf(fp, "col_markdash=%d\n", tinrc.col_markdash);
-	fprintf(fp, "col_markslash=%d\n", tinrc.col_markslash);
+	fprintf(fp, "col_markstar=%d\n\n", tinrc.col_markstar);
+	fprintf(fp, _(txt_col_markdash.tinrc));
+	fprintf(fp, "col_markdash=%d\n\n", tinrc.col_markdash);
+	fprintf(fp, _(txt_col_markslash.tinrc));
+	fprintf(fp, "col_markslash=%d\n\n", tinrc.col_markslash);
+	fprintf(fp, _(txt_col_markstroke.tinrc));
 	fprintf(fp, "col_markstroke=%d\n\n", tinrc.col_markstroke);
 #endif /* HAVE_COLOR */
 
 	fprintf(fp, _(txt_mono_markstar.tinrc));
-	fprintf(fp, "mono_markstar=%d\n", tinrc.mono_markstar);
-	fprintf(fp, "mono_markdash=%d\n", tinrc.mono_markdash);
-	fprintf(fp, "mono_markslash=%d\n", tinrc.mono_markslash);
+	fprintf(fp, "mono_markstar=%d\n\n", tinrc.mono_markstar);
+	fprintf(fp, _(txt_mono_markdash.tinrc));
+	fprintf(fp, "mono_markdash=%d\n\n", tinrc.mono_markdash);
+	fprintf(fp, _(txt_mono_markslash.tinrc));
+	fprintf(fp, "mono_markslash=%d\n\n", tinrc.mono_markslash);
+	fprintf(fp, _(txt_mono_markstroke.tinrc));
 	fprintf(fp, "mono_markstroke=%d\n\n", tinrc.mono_markstroke);
 
 	fprintf(fp, _(txt_mail_address.tinrc));
 	fprintf(fp, "mail_address=%s\n\n", tinrc.mail_address);
 
+#ifdef XFACE_ABLE
+	fprintf(fp, _(txt_use_slrnface.tinrc));
+	fprintf(fp, "use_slrnface=%s\n\n", print_boolean(tinrc.use_slrnface));
+#endif /*XFACE_ABLE */
+
+	fprintf(fp, _(txt_wrap_column.tinrc));
+	fprintf(fp, "wrap_column=%d\n\n", tinrc.wrap_column);
+
 #ifndef CHARSET_CONVERSION
 	fprintf(fp, _(txt_mm_charset.tinrc));
 	fprintf(fp, "mm_charset=%s\n\n", tinrc.mm_charset);
@@ -1388,1117 +1374,6 @@ write_config_file(
 }
 
 
-static int first_option_on_screen;
-static int actual_top_option = 0;
-
-#define option_lines_per_page (cLINES - INDEX_TOP - 3)
-#define TopOfPage(option) option_lines_per_page \
-			* ((option) / option_lines_per_page)
-#define OptionInPage(option)	((option) - first_option_on_screen)
-#define OptionIndex(option)	(OptionInPage(option) % option_lines_per_page)
-
-
-int
-option_row(
-	int option)
-{
-	return (INDEX_TOP + OptionIndex(option));
-}
-
-
-static int
-get_option_num(
-	int act_option)
-{
-	int result = 0;
-
-	if (option_table[act_option].var_type != OPT_TITLE) {
-		while (act_option >= 0) {
-			if (option_table[act_option--].var_type != OPT_TITLE)
-				++result;
-		}
-	}
-	return result;
-}
-
-
-static int
-set_option_num(
-	int option)
-{
-	int result = 0;
-
-	while (option >= 0 && result < LAST_OPT) {
-		while (result < LAST_OPT && option_table[result].var_type == OPT_TITLE)
-			result++;
-		if (result < LAST_OPT) {
-			if (--option < 0)
-				break;
-			++result;
-		}
-	}
-	return result;
-}
-
-
-char *
-fmt_option_prompt(
-	char *dst,
-	size_t len,
-	t_bool editing,
-	int option)
-{
-	char *buf;
-	int num = get_option_num(option);
-	size_t option_width = MAX(35, cCOLS / 2 - 9);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *wbuf, *wbuf2;
-
-	/* convert the option text to wchar_t */
-	wbuf = char2wchar_t(_(option_table[option].txt->opt));
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-
-	if (num) {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		if (wbuf != NULL) {
-			wbuf2 = my_malloc(sizeof(wchar_t) * (option_width + 1));
-			wstrunc(wbuf, wbuf2, option_width + 1, option_width);
-
-			if ((buf = wchar_t2char(wbuf2)) == NULL) {
-				/* conversion failed, truncate original string */
-				buf = my_malloc(option_width + 1);
-				strunc(_(option_table[option].txt->opt), buf, option_width + 1, option_width);
-				snprintf(dst, len, "%s %3d. %-*.*s: ", editing ? "->" : "  ", num, option_width, option_width, buf);
-			} else {
-				snprintf(dst, len, "%s %3d. %-*.*s: ", editing ? "->" : "  ", num,
-					strlen(buf) + option_width - wcswidth(wbuf2, option_width + 1),
-					strlen(buf) + option_width - wcswidth(wbuf2, option_width + 1), buf);
-			}
-			free(wbuf2);
-		} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		{
-			/* truncate original string */
-			buf = my_malloc(option_width + 1);
-			strunc(_(option_table[option].txt->opt), buf, option_width + 1, option_width);
-			snprintf(dst, len, "%s %3d. %-*.*s: ", editing ? "->" : "  ", num, option_width, option_width, buf);
-		}
-	} else {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		if (wbuf != NULL) {
-			wbuf2 = my_malloc(sizeof(wchar_t) * (cCOLS - 3 + 1));
-			wstrunc(wbuf, wbuf2, cCOLS - 3 + 1, cCOLS - 3);
-			if ((buf = wchar_t2char(wbuf2)) == NULL) {
-				/* conversion failed, truncate original string */
-				buf = my_malloc(cCOLS - 3 + 1);
-				strunc(_(option_table[option].txt->opt), buf, cCOLS - 3 + 1, cCOLS - 3);
-			}
-			free(wbuf2);
-		} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		{
-			/* truncate original string */
-			buf = my_malloc(cCOLS - 3 + 1);
-			strunc(_(option_table[option].txt->opt), buf, cCOLS - 3 + 1, cCOLS - 3);
-		}
-		snprintf(dst, len, "  %s", buf);
-	}
-
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	FreeIfNeeded(wbuf);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-	FreeIfNeeded(buf);
-	return dst;
-}
-
-
-static void
-print_any_option(
-	int act_option)
-{
-	constext **list;
-	char temp[LEN], *ptr, *ptr2;
-	int row = option_row(act_option);
-	size_t len = sizeof(temp) - 1;
-
-	MoveCursor(row, 0);
-
-	ptr = fmt_option_prompt(temp, len, FALSE, act_option);
-	ptr += strlen(temp);
-	len -= strlen(temp);
-
-	switch (option_table[act_option].var_type) {
-		case OPT_ON_OFF:
-			/* tailing space to overwrite any left over F from OFF */
-			snprintf(ptr, len, "%s ", print_boolean(*OPT_ON_OFF_list[option_table[act_option].var_index]));
-			break;
-
-		case OPT_LIST:
-			list = option_table[act_option].opt_list;
-			ptr2 = my_strdup(list[*(option_table[act_option].variable) + ((strcasecmp(_(list[0]), _(txt_default)) == 0) ? 1 : 0)]);
-			snprintf(ptr, len, "%s", _(ptr2));
-			free(ptr2);
-			break;
-
-		case OPT_STRING:
-			snprintf(ptr, len, "%s", OPT_STRING_list[option_table[act_option].var_index]);
-			break;
-
-		case OPT_NUM:
-			snprintf(ptr, len, "%d", *(option_table[act_option].variable));
-			break;
-
-		case OPT_CHAR:
-			snprintf(ptr, len, "%c", *OPT_CHAR_list[option_table[act_option].var_index]);
-			break;
-
-		default:
-			break;
-	}
-#ifdef USE_CURSES
-	my_printf("%.*s", cCOLS - 1, temp);
-	{
-		int y, x;
-
-		getyx(stdscr, y, x);
-		if (x < cCOLS)
-			clrtoeol();
-	}
-#else
-	my_printf("%.*s", cCOLS - 1, temp);
-	/* draw_arrow_mark() will read this back for repainting */
-	strncpy(screen[row - INDEX_TOP].col, temp, cCOLS);
-#endif /* USE_CURSES */
-}
-
-
-static void
-print_option(
-	enum option_enum the_option)
-{
-	print_any_option((int) the_option);
-}
-
-
-static t_bool
-OptionOnPage(
-	int option)
-{
-	if ((option >= first_option_on_screen) && (option < first_option_on_screen + option_lines_per_page))
-		return TRUE;
-	return FALSE;
-}
-
-
-static void
-RepaintOption(
-	int option)
-{
-	if (OptionOnPage(option))
-		print_any_option(option);
-}
-
-
-#ifdef USE_CURSES
-static void
-DoScroll(
-	int jump)
-{
-	MoveCursor(INDEX_TOP, 0);
-	SetScrollRegion(INDEX_TOP, INDEX_TOP + option_lines_per_page - 1);
-	ScrollScreen(jump);
-	SetScrollRegion(0, LINES - 1);
-}
-#endif /* USE_CURSES */
-
-
-static void
-highlight_option(
-	int option)
-{
-	if (!OptionOnPage(option)) {
-#ifdef USE_CURSES
-		if (option > 0 && OptionOnPage(option - 1)) {
-			DoScroll(1);
-			first_option_on_screen++;
-		} else if (option < LAST_OPT && OptionOnPage(option + 1)) {
-			DoScroll(-1);
-			first_option_on_screen--;
-		} else
-#endif /* USE_CURSES */
-		{
-			first_option_on_screen = TopOfPage(option);
-			ClearScreen();
-		}
-	}
-
-	refresh_config_page(option);
-	draw_arrow_mark(option_row(option));
-	info_message("%s", _(option_table[option].txt->opt));
-}
-
-
-static void
-unhighlight_option(
-	int option)
-{
-	/* Astonishing hack */
-	t_menu *savemenu = currmenu;
-	t_menu cfgmenu = { 0, 1, 0, 0, NULL, NULL };
-
-	currmenu = &cfgmenu;
-	currmenu->curr = option_row(option) - INDEX_TOP;
-	erase_arrow();
-	currmenu = savemenu;
-	clear_message();
-}
-
-
-/*
- * Refresh the config page which holds the actual option. If act_option is
- * smaller zero fall back on the last given option (first option if there was
- * no last option) and refresh the screen.
- */
-void
-refresh_config_page(
-	int act_option)
-{
-	static int last_option = 0;
-	t_bool force_redraw = FALSE;
-
-	signal_context = cConfig;
-
-	if (act_option < 0) {
-		force_redraw = TRUE;
-		act_option = last_option;
-		ClearScreen();
-	}
-
-	if ((first_option_on_screen != actual_top_option) || force_redraw) {
-		show_config_page();
-		actual_top_option = first_option_on_screen;
-	}
-	last_option = act_option;
-}
-
-
-static void
-redraw_screen(
-	int option)
-{
-	my_retouch();
-	set_xclick_off();
-	ClearScreen();
-	show_config_page();
-	highlight_option(option);
-}
-
-
-/*
- * options menu so that the user can dynamically change parameters
- *
- * TODO: - why do we use ret_code when we never modify it?  what about calling
- *         code which checks the return value?
- *       - when we change something we need to update the related attributes
- *         as well (see line 2009).
- */
-int
-change_config_file(
-	struct t_group *group)
-{
-	int ch = 0;
-	int option, old_option;
-	int ret_code = NO_FILTERING;
-	int mime_encoding = MIME_ENCODING_7BIT;
-	t_bool change_option = FALSE;
-
-	actual_top_option = -1;
-	option = 1;
-
-	ClearScreen();
-	set_xclick_off();
-	forever {
-		highlight_option(option);
-		stow_cursor();
-		ch = ReadCh();
-
-		/*
-		 * convert arrow key codes to "normal" codes
-		 */
-		switch (ch) {
-			case ESC:	/* common arrow keys */
-#	ifdef HAVE_KEY_PREFIX
-			case KEY_PREFIX:
-#	endif /* HAVE_KEY_PREFIX */
-				switch (get_arrow_key(ch)) {
-					case KEYMAP_UP:
-						ch = map_to_local(iKeyUp, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_DOWN:
-						ch = map_to_local(iKeyDown, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_HOME:
-						ch = map_to_local(iKeyFirstPage, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_END:
-						ch = map_to_local(iKeyLastPage, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_PAGE_UP:
-						ch = map_to_local(iKeyPageUp, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_PAGE_DOWN:
-						ch = map_to_local(iKeyPageDown, &menukeymap.config_change);
-						break;
-
-					default:
-						break;
-				} /* switch (get_arrow_key()) */
-				break;
-
-			default:
-				break;
-		}	/* switch (ch) */
-
-		switch (map_to_default(ch, &menukeymap.config_change)) {
-			case iKeyQuit:
-				write_config_file(local_config_file);
-				/* FALLTHROUGH */
-			case iKeyConfigNoSave:
-				clear_note_area();
-				return ret_code;
-
-			case iKeyUp:
-			case iKeyUp2:
-				unhighlight_option(option);
-				if (!get_option_num(--option))	/* skip over titles */
-					option--;
-				if (option < 0)
-					option = LAST_OPT;
-				highlight_option(option);
-				break;
-
-			case iKeyDown:
-			case iKeyDown2:
-				unhighlight_option(option);
-				if (!get_option_num(++option))	/* skip over titles */
-					option++;
-				if (option > LAST_OPT)
-					option = 1;
-				highlight_option(option);
-				break;
-
-			case iKeyFirstPage:
-			case iKeyConfigFirstPage2:
-				unhighlight_option(option);
-				option = 1;
-				highlight_option(option);
-				break;
-
-			case iKeyLastPage:
-			case iKeyConfigLastPage2:
-				unhighlight_option(option);
-				option = LAST_OPT;
-				highlight_option(option);
-				break;
-
-			case iKeyPageUp:
-			case iKeyPageUp2:
-			case iKeyPageUp3:
-				unhighlight_option(option);
-				if (OptionInPage(option)) {
-					option = first_option_on_screen;
-				} else if (!first_option_on_screen) {
-					option = LAST_OPT;
-					first_option_on_screen = TopOfPage(option);
-					ClearScreen();
-					show_config_page();
-				} else if ((option -= option_lines_per_page) < 0) {
-					option = 1;
-					first_option_on_screen = 0;
-				} else {
-					first_option_on_screen -= (tinrc.scroll_lines == -2) ? option_lines_per_page / 2 : option_lines_per_page;
-					ClearScreen();
-					show_config_page();
-				}
-				highlight_option(option);
-				break;
-
-			case iKeyPageDown:
-			case iKeyPageDown2:
-			case iKeyPageDown3:
-				unhighlight_option(option);
-				first_option_on_screen += (tinrc.scroll_lines == -2) ? option_lines_per_page / 2 : option_lines_per_page;
-				if (first_option_on_screen > LAST_OPT)
-					first_option_on_screen = 0;
-
-				option = first_option_on_screen;
-				ClearScreen();
-				show_config_page();
-				highlight_option(option);
-				break;
-
-			case '1': case '2': case '3': case '4': case '5':
-			case '6': case '7': case '8': case '9':
-				unhighlight_option(option);
-				old_option = option;
-				option = prompt_num(ch, _(txt_enter_option_num)) - 1;
-				option = set_option_num(option);
-				if (option < 0 || option > LAST_OPT) {
-					option = old_option;
-					break;
-				}
-				highlight_option(option);
-				break;
-
-			case iKeySearchSubjF:
-			case iKeySearchSubjB:
-			case iKeySearchRepeat:
-				if (ch == iKeySearchRepeat && i_key_search_last != iKeySearchSubjF && i_key_search_last != iKeySearchSubjB)
-					break;
-
-				old_option = option;
-				option = search_config((ch == iKeySearchSubjF), (ch == iKeySearchRepeat), option, LAST_OPT);
-				if (option != old_option) {
-					unhighlight_option(old_option);
-					highlight_option(option);
-				}
-				break;
-
-			case iKeyConfigSelect:
-			case iKeyConfigSelect2:
-				change_option = TRUE;
-				break;
-
-			case iKeyRedrawScr:	/* redraw screen */
-				redraw_screen(option);
-				break;
-
-			default:
-				break;
-		} /* switch (ch) */
-
-		if (change_option) {
-			switch (option_table[option].var_type) {
-				case OPT_ON_OFF:
-					switch (option) {
-						case OPT_ADD_POSTED_TO_FILTER:
-						case OPT_ADVERTISING:
-						case OPT_ALTERNATIVE_HANDLING:
-						case OPT_ASK_FOR_METAMAIL:
-						case OPT_AUTO_BCC:
-						case OPT_AUTO_CC:
-						case OPT_AUTO_LIST_THREAD:
-						case OPT_AUTO_RECONNECT:
-						case OPT_AUTO_SAVE:
-						case OPT_BATCH_SAVE:
-						case OPT_CACHE_OVERVIEW_FILES:
-						case OPT_CATCHUP_READ_GROUPS:
-						case OPT_FORCE_SCREEN_REDRAW:
-						case OPT_GROUP_CATCHUP_ON_EXIT:
-						case OPT_KEEP_DEAD_ARTICLES:
-						case OPT_MARK_IGNORE_TAGS:
-						case OPT_MARK_SAVED_READ:
-						case OPT_PGDN_GOTO_NEXT:
-						case OPT_POS_FIRST_UNREAD:
-						case OPT_POST_PROCESS_VIEW:
-						case OPT_PRINT_HEADER:
-						case OPT_PROCESS_ONLY_UNREAD:
-						case OPT_PROMPT_FOLLOWUPTO:
-						case OPT_SHOW_ONLY_UNREAD_GROUPS:
-						case OPT_SHOW_SIGNATURES:
-						case OPT_SIGDASHES:
-						case OPT_SIGNATURE_REPOST:
-						case OPT_SPACE_GOTO_NEXT_UNREAD:
-						case OPT_START_EDITOR_OFFSET:
-						case OPT_STRIP_BLANKS:
-						case OPT_STRIP_NEWSRC:
-						case OPT_TAB_GOTO_NEXT_UNREAD:
-						case OPT_TEX2ISO_CONV:
-						case OPT_THREAD_CATCHUP_ON_EXIT:
-#if defined(HAVE_ICONV_OPEN_TRANSLIT) && defined(CHARSET_CONVERSION)
-						case OPT_TRANSLIT:
-#endif /* HAVE_ICONV_OPEN_TRANSLIT && CHARSET_CONVERSION */
-						case OPT_UNLINK_ARTICLE:
-						case OPT_URL_HIGHLIGHT:
-#ifdef HAVE_KEYPAD
-						case OPT_USE_KEYPAD:
-#endif /* HAVE_KEYPAD */
-						case OPT_USE_MOUSE:
-						case OPT_WRAP_ON_NEXT_UNREAD:
-							prompt_option_on_off(option);
-							break;
-
-						/* show mini help menu */
-						case OPT_BEGINNER_LEVEL:
-							if (prompt_option_on_off(option))
-								set_noteslines(cLINES);
-							break;
-
-						/* show all arts or just new/unread arts */
-						case OPT_SHOW_ONLY_UNREAD_ARTS:
-							if (prompt_option_on_off(option) && group != NULL) {
-								make_threads(group, TRUE);
-								pos_first_unread_thread();
-							}
-							break;
-
-						/* draw -> / highlighted bar */
-						case OPT_DRAW_ARROW:
-							prompt_option_on_off(option);
-							unhighlight_option(option);
-							if (!tinrc.draw_arrow && !tinrc.inverse_okay) {
-								tinrc.inverse_okay = TRUE;
-								RepaintOption(OPT_INVERSE_OKAY);
-							}
-							break;
-
-						/* draw inversed screen header lines */
-						/* draw inversed group/article/option line if draw_arrow is OFF */
-						case OPT_INVERSE_OKAY:
-							prompt_option_on_off(option);
-							unhighlight_option(option);
-							if (!tinrc.draw_arrow && !tinrc.inverse_okay) {
-								tinrc.draw_arrow = TRUE;	/* we don't want to navigate blindly */
-								RepaintOption(OPT_DRAW_ARROW);
-							}
-							break;
-
-						case OPT_MAIL_8BIT_HEADER:
-							prompt_option_on_off(option);
-							if (strcasecmp(txt_mime_encodings[tinrc.mail_mime_encoding], txt_8bit)) {
-								tinrc.mail_8bit_header = FALSE;
-								print_option(OPT_MAIL_8BIT_HEADER);
-							}
-							break;
-
-						case OPT_POST_8BIT_HEADER:
-							prompt_option_on_off(option);
-							/* if post_mime_encoding != 8bit, post_8bit_header is disabled */
-							if (strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_8bit)) {
-								tinrc.post_8bit_header = FALSE;
-								print_option(OPT_POST_8BIT_HEADER);
-							}
-							break;
-
-						/* show newsgroup description text next to newsgroups */
-						case OPT_SHOW_DESCRIPTION:
-							prompt_option_on_off(option);
-							show_description = tinrc.show_description;
-							if (show_description)			/* force reread of newgroups file */
-								read_descriptions(FALSE);
-							break;
-
-#ifdef HAVE_COLOR
-						/* use ANSI color */
-						case OPT_USE_COLOR:
-							prompt_option_on_off(option);
-#	ifdef USE_CURSES
-							if (!has_colors())
-								use_color = FALSE;
-							else
-#	endif /* USE_CURSES */
-								use_color = tinrc.use_color;
-							break;
-#endif /* HAVE_COLOR */
-
-#ifdef XFACE_ABLE
-						/* use slrnface */
-						case OPT_USE_SLRNFACE:
-							if (prompt_option_on_off(option)) {
-								if (!tinrc.use_slrnface)
-									slrnface_stop();
-								else
-									slrnface_start();
-							}
-							break;
-#endif /* XFACE_ABLE */
-
-						/* word_highlight */
-						case OPT_WORD_HIGHLIGHT:
-							prompt_option_on_off(option);
-							word_highlight = tinrc.word_highlight;
-							break;
-
-#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
-						case OPT_RENDER_BIDI:
-							prompt_option_on_off(option);
-							break;
-#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
-
-						default:
-							break;
-					} /* switch (option) */
-					break;
-
-				case OPT_LIST:
-					switch (option) {
-#ifdef HAVE_COLOR
-						case OPT_COL_BACK:
-						case OPT_COL_FROM:
-						case OPT_COL_HEAD:
-						case OPT_COL_HELP:
-						case OPT_COL_INVERS_BG:
-						case OPT_COL_INVERS_FG:
-						case OPT_COL_MESSAGE:
-						case OPT_COL_MINIHELP:
-						case OPT_COL_NEWSHEADERS:
-						case OPT_COL_NORMAL:
-						case OPT_COL_QUOTE:
-						case OPT_COL_QUOTE2:
-						case OPT_COL_QUOTE3:
-						case OPT_COL_RESPONSE:
-						case OPT_COL_SIGNATURE:
-						case OPT_COL_SUBJECT:
-						case OPT_COL_TEXT:
-						case OPT_COL_TITLE:
-						case OPT_COL_MARKSTAR:
-						case OPT_COL_MARKDASH:
-						case OPT_COL_MARKSLASH:
-						case OPT_COL_MARKSTROKE:
-						case OPT_COL_URLS:
-#endif /* HAVE_COLOR */
-						case OPT_HIDE_UUE:
-						case OPT_INTERACTIVE_MAILER:
-						case OPT_WORD_H_DISPLAY_MARKS:
-						case OPT_MONO_MARKSTAR:
-						case OPT_MONO_MARKDASH:
-						case OPT_MONO_MARKSLASH:
-						case OPT_MONO_MARKSTROKE:
-						case OPT_CONFIRM_CHOICE:
-						case OPT_KILL_LEVEL:
-						case OPT_MAILBOX_FORMAT:
-						case OPT_SHOW_INFO:
-						case OPT_SORT_ARTICLE_TYPE:
-						case OPT_STRIP_BOGUS:
-#ifdef HAVE_UNICODE_NORMALIZATION
-						case OPT_NORMALIZATION_FORM:
-#endif /* HAVE_UNICODE_NORMALIZATION */
-						case OPT_QUOTE_STYLE:
-						case OPT_WILDCARD:
-							prompt_option_list(option);
-							break;
-
-						case OPT_THREAD_ARTICLES:
-							/*
-							 * If the threading strategy has changed, fix things
-							 * so that rethreading will occur
-							 */
-							if (prompt_option_list(option) && group != NULL) {
-								int n, old_base_art = base[grpmenu.curr];
-
-								group->attribute->thread_arts = tinrc.thread_articles;
-								make_threads(group, TRUE);
-								/* in non-empty groups update cursor position */
-								if (grpmenu.max > 0) {
-									if ((n = which_thread(old_base_art)) >= 0)
-										grpmenu.curr = n;
-								}
-							}
-							clear_message();
-							break;
-
-						case OPT_SORT_THREADS_TYPE:
-							/*
-							 * If the sorting strategy of threads has changed, fix things
-							 * so that resorting will occur
-							 */
-							if (prompt_option_list(option) && group != NULL) {
-								group->attribute->sort_threads_type = tinrc.sort_threads_type;
-								make_threads(group, TRUE);
-							}
-							clear_message();
-							break;
-
-						case OPT_THREAD_SCORE:
-							/*
-							 * If the scoring of a thread has changed,
-							 * resort base[]
-							 */
-							if (prompt_option_list(option) && group != NULL)
-								find_base(group);
-							clear_message();
-							break;
-
-						case OPT_POST_PROCESS:
-							prompt_option_list(option);
-							glob_attributes.post_proc_type = tinrc.post_process;
-							if (group != NULL)
-								group->attribute->post_proc_type = tinrc.post_process;
-							break;
-
-						case OPT_SHOW_AUTHOR:
-							prompt_option_list(option);
-							if (group != NULL)
-								group->attribute->show_author = tinrc.show_author;
-							break;
-
-						case OPT_MAIL_MIME_ENCODING:
-						case OPT_POST_MIME_ENCODING:
-							prompt_option_list(option);
-							mime_encoding = *(option_table[option].variable);
-							/* do not use 8 bit headers if mime encoding is not 8bit */
-							if (strcasecmp(txt_mime_encodings[mime_encoding], txt_8bit)) {
-								if (option == (int) OPT_POST_MIME_ENCODING) {
-									tinrc.post_8bit_header = FALSE;
-									RepaintOption(OPT_POST_8BIT_HEADER);
-								} else {
-									tinrc.mail_8bit_header = FALSE;
-									RepaintOption(OPT_MAIL_8BIT_HEADER);
-								}
-							}
-							break;
-
-#ifdef CHARSET_CONVERSION
-						case OPT_MM_NETWORK_CHARSET:
-							if (prompt_option_list(option)) {
-								glob_attributes.mm_network_charset = tinrc.mm_network_charset;
-								if (group)
-									group->attribute->mm_network_charset = tinrc.mm_network_charset;
-#	ifdef NO_LOCALE
-								strcpy(tinrc.mm_local_charset, txt_mime_charsets[tinrc.mm_network_charset]);
-#	endif /* NO_LOCALE */
-							}
-							/*
-							 * check if we have selected a 7bit charset, otherwise
-							 * update encoding
-							 * we always do this (even if we did not change the
-							 * charset) to fixup flaws in the tinrc - once we do
-							 * the same while reading the tinrc this can go into
-							 * the != original_list_value case.
-							 */
-							{
-								int i;
-								t_bool change;
-
-								if (!strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)) {
-									change = TRUE;
-									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
-										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
-											change = FALSE;
-											break;
-										}
-									}
-									if (change) {
-										tinrc.post_mime_encoding = MIME_ENCODING_8BIT;
-										RepaintOption(OPT_POST_MIME_ENCODING);
-									}
-								} else { /* and vice versa, if we have a 7bit chaset but a !7bit encoding, fix that */
-									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
-										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
-											tinrc.mail_mime_encoding = tinrc.post_mime_encoding = MIME_ENCODING_7BIT;
-											tinrc.mail_8bit_header = tinrc.post_8bit_header = FALSE;
-											RepaintOption(OPT_POST_MIME_ENCODING);
-											RepaintOption(OPT_MAIL_MIME_ENCODING);
-											RepaintOption(OPT_POST_8BIT_HEADER);
-											break;
-										}
-									}
-								}
-
-								if (!strcasecmp(txt_mime_encodings[tinrc.mail_mime_encoding], txt_7bit)) {
-									change = TRUE;
-									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
-										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
-											change = FALSE;
-											break;
-										}
-									}
-									if (change) {
-										tinrc.mail_mime_encoding = MIME_ENCODING_QP;
-										RepaintOption(OPT_MAIL_MIME_ENCODING);
-									}
-								} else { /* and vice versa, if we have a 7bit chaset but a !7bit encoding, fix that */
-									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
-										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
-											tinrc.mail_mime_encoding = tinrc.post_mime_encoding = MIME_ENCODING_7BIT;
-											tinrc.mail_8bit_header = tinrc.post_8bit_header = FALSE;
-											RepaintOption(OPT_POST_MIME_ENCODING);
-											RepaintOption(OPT_MAIL_MIME_ENCODING);
-											RepaintOption(OPT_POST_8BIT_HEADER);
-											break;
-										}
-									}
-								}
-							}
-							break;
-#endif /* CHARSET_CONVERSION */
-
-						default:
-							break;
-					} /* switch (option) */
-					break;
-
-				case OPT_STRING:
-					switch (option) {
-						case OPT_EDITOR_FORMAT:
-						case OPT_INEWS_PROG:
-						case OPT_MAILER_FORMAT:
-						case OPT_MAIL_ADDRESS:
-						case OPT_MAIL_QUOTE_FORMAT:
-						case OPT_METAMAIL_PROG:
-#ifndef CHARSET_CONVERSION
-						case OPT_MM_CHARSET:
-#endif /* !CHARSET_CONVERSION */
-						case OPT_NEWS_QUOTE_FORMAT:
-						case OPT_QUOTE_CHARS:
-						case OPT_SPAMTRAP_WARNING_ADDRESSES:
-						case OPT_URL_HANDLER:
-						case OPT_XPOST_QUOTE_FORMAT:
-							prompt_option_string(option);
-							break;
-
-						case OPT_NEWS_HEADERS_TO_DISPLAY:
-							prompt_option_string(option);
-							if (news_headers_to_display_array)
-								FreeIfNeeded(*news_headers_to_display_array);
-							FreeIfNeeded(news_headers_to_display_array);
-							news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
-							break;
-
-						case OPT_NEWS_HEADERS_TO_NOT_DISPLAY:
-							prompt_option_string(option);
-							if (news_headers_to_not_display_array)
-								FreeIfNeeded(*news_headers_to_not_display_array);
-							FreeIfNeeded(news_headers_to_not_display_array);
-							news_headers_to_not_display_array = ulBuildArgv(tinrc.news_headers_to_not_display, &num_headers_to_not_display);
-							break;
-
-#ifndef DISABLE_PRINTING
-						case OPT_PRINTER:
-#endif /* !DISABLE_PRINTING */
-						case OPT_MAILDIR:
-						case OPT_SAVEDIR:
-						case OPT_SIGFILE:
-						case OPT_POSTED_ARTICLES_FILE:
-							if (prompt_option_string(option)) {
-								char buf[PATH_LEN];
-
-								strfpath(tinrc.posted_articles_file, buf, sizeof(buf), &CURR_GROUP);
-								STRCPY(tinrc.posted_articles_file, buf);
-							}
-							break;
-
-#ifdef HAVE_COLOR
-						case OPT_QUOTE_REGEX:
-							prompt_option_string(option);
-							FreeIfNeeded(quote_regex.re);
-							FreeIfNeeded(quote_regex.extra);
-							if (!strlen(tinrc.quote_regex))
-								STRCPY(tinrc.quote_regex, DEFAULT_QUOTE_REGEX);
-							compile_regex(tinrc.quote_regex, &quote_regex, PCRE_CASELESS);
-							break;
-
-						case OPT_QUOTE_REGEX2:
-							prompt_option_string(option);
-							FreeIfNeeded(quote_regex2.re);
-							FreeIfNeeded(quote_regex2.extra);
-							if (!strlen(tinrc.quote_regex2))
-								STRCPY(tinrc.quote_regex2, DEFAULT_QUOTE_REGEX2);
-							compile_regex(tinrc.quote_regex2, &quote_regex2, PCRE_CASELESS);
-							break;
-
-						case OPT_QUOTE_REGEX3:
-							prompt_option_string(option);
-							FreeIfNeeded(quote_regex3.re);
-							FreeIfNeeded(quote_regex3.extra);
-							if (!strlen(tinrc.quote_regex3))
-								STRCPY(tinrc.quote_regex3, DEFAULT_QUOTE_REGEX3);
-							compile_regex(tinrc.quote_regex3, &quote_regex3, PCRE_CASELESS);
-							break;
-#endif /* HAVE_COLOR */
-
-						case OPT_SLASHES_REGEX:
-							prompt_option_string(option);
-							FreeIfNeeded(slashes_regex.re);
-							FreeIfNeeded(slashes_regex.extra);
-							if (!strlen(tinrc.slashes_regex))
-								STRCPY(tinrc.slashes_regex, DEFAULT_SLASHES_REGEX);
-							compile_regex(tinrc.slashes_regex, &slashes_regex, PCRE_CASELESS);
-							break;
-
-						case OPT_STARS_REGEX:
-							prompt_option_string(option);
-							FreeIfNeeded(stars_regex.re);
-							FreeIfNeeded(stars_regex.extra);
-							if (!strlen(tinrc.stars_regex))
-								STRCPY(tinrc.stars_regex, DEFAULT_STARS_REGEX);
-							compile_regex(tinrc.stars_regex, &stars_regex, PCRE_CASELESS);
-							break;
-
-						case OPT_STROKES_REGEX:
-							prompt_option_string(option);
-							FreeIfNeeded(strokes_regex.re);
-							FreeIfNeeded(strokes_regex.extra);
-							if (!strlen(tinrc.strokes_regex))
-								STRCPY(tinrc.strokes_regex, DEFAULT_STROKES_REGEX);
-							compile_regex(tinrc.strokes_regex, &strokes_regex, PCRE_CASELESS);
-							break;
-
-						case OPT_UNDERSCORES_REGEX:
-							prompt_option_string(option);
-							FreeIfNeeded(underscores_regex.re);
-							FreeIfNeeded(underscores_regex.extra);
-							if (!strlen(tinrc.underscores_regex))
-								STRCPY(tinrc.underscores_regex, DEFAULT_UNDERSCORES_REGEX);
-							compile_regex(tinrc.underscores_regex, &underscores_regex, PCRE_CASELESS);
-							break;
-
-						case OPT_STRIP_RE_REGEX:
-							prompt_option_string(option);
-							FreeIfNeeded(strip_re_regex.re);
-							FreeIfNeeded(strip_re_regex.extra);
-							if (!strlen(tinrc.strip_re_regex))
-								STRCPY(tinrc.strip_re_regex, DEFAULT_STRIP_RE_REGEX);
-							compile_regex(tinrc.strip_re_regex, &strip_re_regex, PCRE_ANCHORED);
-							break;
-
-						case OPT_STRIP_WAS_REGEX:
-							prompt_option_string(option);
-							FreeIfNeeded(strip_was_regex.re);
-							FreeIfNeeded(strip_was_regex.extra);
-							if (!strlen(tinrc.strip_was_regex)) {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-								if (IS_LOCAL_CHARSET("UTF-8")) {
-#	if (defined(PCRE_MAJOR) && PCRE_MAJOR >= 4)
-									int i;
-
-									pcre_config(PCRE_CONFIG_UTF8, &i);
-									if (i)
-										STRCPY(tinrc.strip_was_regex, DEFAULT_U8_STRIP_WAS_REGEX);
-									else
-#	endif /* PCRE_MAJOR && PCRE_MAJOR >=4 */
-										STRCPY(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX);
-								} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-									STRCPY(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX);
-							}
-							compile_regex(tinrc.strip_was_regex, &strip_was_regex, 0);
-							break;
-
-						case OPT_DATE_FORMAT:
-							prompt_option_string(option);
-							if (!strlen(tinrc.date_format)) {
-								STRCPY(tinrc.date_format, DEFAULT_DATE_FORMAT);
-							}
-							break;
-
-						default:
-							break;
-					} /* switch (option) */
-
-					break;
-
-				case OPT_NUM:
-					switch (option) {
-						case OPT_GETART_LIMIT:
-						case OPT_SCROLL_LINES:
-							prompt_option_num(option);
-							break;
-
-						case OPT_REREAD_ACTIVE_FILE_SECS:
-							prompt_option_num(option);
-							if (tinrc.reread_active_file_secs < 0)
-								tinrc.reread_active_file_secs = 0;
-							break;
-
-						case OPT_RECENT_TIME:
-							prompt_option_num(option);
-							if (tinrc.recent_time < 0)
-								tinrc.recent_time = 0;
-							break;
-
-						case OPT_GROUPNAME_MAX_LENGTH:
-							prompt_option_num(option);
-							if (tinrc.groupname_max_length < 0)
-								tinrc.groupname_max_length = 0;
-							break;
-
-						case OPT_FILTER_DAYS:
-							prompt_option_num(option);
-							if (tinrc.filter_days <= 0)
-								tinrc.filter_days = 1;
-							break;
-
-						case OPT_SCORE_LIMIT_KILL:
-						case OPT_SCORE_KILL:
-						case OPT_SCORE_LIMIT_SELECT:
-						case OPT_SCORE_SELECT:
-							prompt_option_num(option);
-							check_score_defaults();
-							if (group != NULL) {
-								unfilter_articles();
-								read_filter_file(filter_file);
-								if (filter_articles(group))
-									make_threads(group, FALSE);
-							}
-							redraw_screen(option);
-							break;
-
-						case OPT_WRAP_COLUMN:
-							prompt_option_num(option);
-							/* recook if in an article is open */
-							if (pgart.raw)
-								resize_article(TRUE, &pgart);
-							break;
-
-						default:
-							break;
-					} /* switch (option) */
-					break;
-
-				case OPT_CHAR:
-					switch (option) {
-						/*
-						 * TODO: do DASH_TO_SPACE/SPACE_TO_DASH conversion here?
-						 */
-						case OPT_ART_MARKED_DELETED:
-						case OPT_ART_MARKED_INRANGE:
-						case OPT_ART_MARKED_RETURN:
-						case OPT_ART_MARKED_SELECTED:
-						case OPT_ART_MARKED_RECENT:
-						case OPT_ART_MARKED_UNREAD:
-						case OPT_ART_MARKED_READ:
-						case OPT_ART_MARKED_KILLED:
-						case OPT_ART_MARKED_READ_SELECTED:
-							prompt_option_char(option);
-							break;
-
-						default:
-							break;
-					} /* switch (option) */
-					break;
-
-				default:
-					break;
-			} /* switch (option_table[option].var_type) */
-			change_option = FALSE;
-			show_menu_help(txt_select_config_file_option);
-			RepaintOption(option);
-		} /* if (change_option) */
-	} /* forever */
-	/* NOTREACHED */
-	return ret_code;
-}
-
-
-/*
- * show_menu_help
- */
-void
-show_menu_help(
-	const char *help_message)
-{
-	MoveCursor(cLINES - 2, 0);
-	CleartoEOLN();
-	center_line(cLINES - 2, FALSE, _(help_message));
-}
-
-
 t_bool
 match_boolean(
 	char *line,
@@ -2639,10 +1514,11 @@ match_string(
 	size_t patlen = strlen(pat);
 
 	if (STRNCMPEQ(line, pat, patlen) && (strlen(line) > patlen /* + 1 */)) {
-		strncpy(dst, &line[patlen], dstlen);
-		if ((ptr = strrchr(dst, '\n')) != NULL)
-			*ptr = '\0';
-
+		if (dst != NULL && dstlen >= 1) {
+			strncpy(dst, &line[patlen], dstlen);
+			if ((ptr = strrchr(dst, '\n')) != NULL)
+				*ptr = '\0';
+		}
 		return TRUE;
 	}
 	return FALSE;
@@ -2724,33 +1600,6 @@ quote_space_to_dash(
 }
 
 
-/*
- * display current configuration page
- */
-static void
-show_config_page(
-	void)
-{
-	int i, lines_to_print = option_lines_per_page;
-
-	center_line(0, TRUE, _(txt_options_menu));
-
-	/*
-	 * on last page, there need not be option_lines_per_page options
-	 */
-	if (first_option_on_screen + option_lines_per_page > LAST_OPT)
-		lines_to_print = LAST_OPT + 1 - first_option_on_screen;
-
-	for (i = 0; i < lines_to_print; i++)
-		print_any_option(first_option_on_screen + i);
-	CleartoEOS();
-
-	show_menu_help(txt_select_config_file_option);
-	my_flush();
-	stow_cursor();
-}
-
-
 /*
  * Written by: Brad Viviano and Scott Powers (bcv & swp)
  *
@@ -2812,21 +1661,6 @@ ulBuildArgv(
 }
 
 
-/*
- * Check if score_kill is <= score_limit_kill and if score_select >= score_limit_select
- */
-static void
-check_score_defaults(
-	void)
-{
-	if (tinrc.score_kill > tinrc.score_limit_kill)
-		tinrc.score_kill = tinrc.score_limit_kill;
-
-	if (tinrc.score_select < tinrc.score_limit_select)
-		tinrc.score_select = tinrc.score_limit_select;
-}
-
-
 /*
  * auto update tinrc
  */
@@ -2927,7 +1761,7 @@ rc_update(
 	}
 
 	/* update the values */
-	tinrc.confirm_choice = (confirm_action ? 1 : 0 ) + (confirm_to_quit ? 3 : 0);
+	tinrc.confirm_choice = (confirm_action ? 1 : 0) + (confirm_to_quit ? 3 : 0);
 
 	if (!use_getart_limit)
 		tinrc.getart_limit = 0;
@@ -2976,7 +1810,6 @@ read_server_config(
 	char file[PATH_LEN];
 	char newnews_info[LEN];
 	char serverdir[PATH_LEN];
-	char version[LEN];
 	int upgrade = RC_CHECK;
 
 #ifdef NNTP_ABLE
@@ -2997,7 +1830,7 @@ read_server_config(
 			continue;
 
 		if (match_string(line, "last_newnews=", newnews_info, sizeof(newnews_info))) {
-			int tmp_len = strlen(nntp_server) + strlen(newnews_info) + 2;
+			size_t tmp_len = strlen(nntp_server) + strlen(newnews_info) + 2;
 			char *tmp_info = my_malloc(tmp_len);
 
 			snprintf(tmp_info, tmp_len, "%s %s", nntp_server, newnews_info);
@@ -3005,7 +1838,7 @@ read_server_config(
 			free(tmp_info);
 			continue;
 		}
-		if (match_string(line, "version=", version, sizeof(version))) {
+		if (match_string(line, "version=", NULL, 0)) {
 			if (RC_CHECK != upgrade)
 				/* ignore duplicate version lines; last match counts */
 				continue;
