@@ -6,7 +6,7 @@
  *  Updated   : 2003-09-19
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
 
  * All rights reserved.
  *
@@ -68,7 +68,7 @@ hash_str(
 	struct t_hashnode **p;	/* used to descend the spillover structs */
 
 	if (s == NULL)
-		return (char *) 0;
+		return NULL;
 
 	h = 0;
 	while (*t) {
