@@ -5,7 +5,7 @@
  *  Created   : 1997-03-10
  *  Updated   : 2003-09-19
  *
- * Copyright (c) 1997-2004 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2005 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -372,9 +372,9 @@ build_sender(
 			strcat(sender, ptr);
 			strcat(sender, ">");
 		} else
-			return (char *) 0;
+			return NULL;
 	} else
-		return (char *) 0;
+		return NULL;
 
 	return sender;
 }
