@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2004-06-06
+ *  Updated   : 2005-01-30
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,9 +44,9 @@
 #ifndef included_trace_h
 #	include "trace.h"
 #endif /* !included_trace_h */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
 #ifndef VERSION_H
 #	include "version.h"
 #endif /* !VERSION_H */
@@ -124,9 +124,9 @@ char userid[PATH_LEN];
 	char rcdir_asfile[PATH_LEN];	/* rcdir expressed as dev:[dir]tin.dir, for stat() */
 #endif /* VMS */
 
+t_function last_search;	/* for repeated search */
 int hist_last[HIST_MAXNUM + 1];
 int hist_pos[HIST_MAXNUM + 1];
-int i_key_search_last;			/* for repeated search */
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
 int num_headers_to_display;		/* num headers to display -- swp */
 int num_headers_to_not_display;		/* num headers to not display -- swp */
@@ -275,7 +275,7 @@ struct t_config tinrc = {
 	FILTER_SUBJ_CASE_SENSITIVE,		/* default_filter_kill_header */
 	FILTER_SUBJ_CASE_SENSITIVE,		/* default_filter_select_header */
 	0,		/* default_move_group */
-	iKeySaveAppendFile,		/* default_save_mode */
+	'a',		/* default_save_mode */
 	0,		/* getart_limit */
 	2,		/* recent_time */
 	32,		/* groupname_max_length */
@@ -510,14 +510,19 @@ init_selfinfo(
 
 	domain_name[0] = '\0';
 
-#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
-	if (uname(&system_info) < 0) {
+#ifdef HAVE_SYS_UTSNAME_H
+#	ifdef HAVE_UNAME
+	if (uname(&system_info) != -1)
+		;
+	else
+#	endif /* HAVE_UNAME */
+	{
 		strcpy(system_info.sysname, "unknown");
 		*system_info.machine = '\0';
 		*system_info.release = '\0';
 		*system_info.nodename = '\0';
 	}
-#endif /* HAVE_SYS_UTSNAME_H && HAVE_UNAME */
+#endif /* HAVE_SYS_UTSNAME_H */
 
 #ifdef DOMAIN_NAME
 	if ((cptr = get_domain_name()) != NULL)
@@ -527,7 +532,7 @@ init_selfinfo(
 #ifdef HAVE_GETHOSTBYNAME
 	if (domain_name[0] == '\0') {
 		cptr = get_fqdn(get_host_name());
-		if (cptr != (char *) 0)
+		if (cptr != NULL)
 			strcpy(domain_name, cptr);
 	}
 #endif /* HAVE_GETHOSTBYNAME */
@@ -608,7 +613,7 @@ init_selfinfo(
 	newsrc[0] = '\0';
 
 	snprintf(page_header, sizeof(page_header), "%s %s release %s (\"%s\") [%s%s]",
-		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME,
+		PRODUCT, VERSION, RELEASEDATE, RELEASENAME, OSNAME,
 		(iso2asc_supported >= 0 ? " ISO2ASC" : ""));
 	snprintf(cvers, sizeof(cvers), txt_copyright_notice, page_header);
 
@@ -650,11 +655,6 @@ init_selfinfo(
 	newsgroups_file[0] = '\0';
 	subscriptions_file[0] = '\0';
 
-	/*
-	 * Setup default keymaps
-	 */
-	build_keymaps();
-
 	/*
 	 * read the global site config file to override some default
 	 * values given at compile time
@@ -991,6 +991,6 @@ utf8_pcre(
 	(void) pcre_config(PCRE_CONFIG_UTF8, &i);
 #	endif /* PCRE_MAJOR && PCRE_MAJOR >= $*/
 
-	return (i ? TRUE: FALSE);
+	return (i ? TRUE : FALSE);
 }
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
