@@ -3,10 +3,10 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-01-12
+ *  Updated   : 2005-03-12
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -209,20 +209,21 @@ main(
 	}
 #endif /* DEBUG_NEWSRC */
 
+	/*
+	 * avoid empty regexp, we also need to do this in batch_mode
+	 * as read_overview() calls eat_re() which uses a regexp to
+	 * modify the subject *sigh*
+	 */
+	postinit_regexp();
+
 	if (!(batch_mode || post_postponed_and_exit)) {
 		/*
 		 * Read user specific keybindings and input history
 		 */
 		wait_message(0, _(txt_reading_keymap_file));
-		if (!read_keymap_file())
-			prompt_continue();
+		read_keymap_file();
 		read_input_history_file();
 
-		/*
-		 * avoid empty regexp
-		 */
-		postinit_regexp();
-
 		/*
 		 * Load the mail & news active files into active[]
 		 *
@@ -586,230 +587,7 @@ read_cmd_line_options(
 				break;
 
 			case 'V':
-#if defined(__DATE__) && defined(__TIME__)
-				error_message(_("Version: %s release %s (\"%s\") %s %s"),
-					VERSION, RELEASEDATE, RELEASENAME, __DATE__, __TIME__);
-#else
-				error_message(_("Version: %s release %s (\"%s\")"),
-					VERSION, RELEASEDATE, RELEASENAME);
-#endif /* __DATE__ && __TIME__ */
-/*
- * FIXME: make the rest below a function (i.e. to fill a struct), so we
- * can also use it inside post.c:mail_bug_report()
- */
-#ifdef SYSTEM_NAME
-				error_message("Platform:");
-				error_message("\tOS-Name  = \"%s\"", SYSTEM_NAME);
-#endif /* SYSTEM_NAME */
-#ifdef TIN_CC
-				error_message("Compiler:");
-				error_message("\tCC       = \"%s\"", TIN_CC);
-#	ifdef TIN_CFLAGS
-				error_message("\tCFLAGS   = \"%s\"", TIN_CFLAGS);
-#	endif /* TIN_CFLAGS */
-#	ifdef TIN_CPP
-				error_message("\tCPP      = \"%s\"", TIN_CPP);
-#	endif /* TIN_CPP */
-#	ifdef TIN_CPPFLAGS
-				error_message("\tCPPFLAGS = \"%s\"", TIN_CPPFLAGS);
-#	endif /* TIN_CPPFLAGS */
-#endif /* TIN_CC */
-
-#ifdef TIN_LD
-				error_message("Linker and Libraries:");
-				error_message("\tLD       = \"%s\"", TIN_LD);
-#	ifdef TIN_LDFLAGS
-				error_message("\tLDFLAGS  = \"%s\"", TIN_LDFLAGS);
-#	endif /* TIN_LDFLAGS */
-
-#	ifdef TIN_LIBS
-				error_message("\tLIBS     = \"%s\"", TIN_LIBS);
-#	endif /* TIN_LIBS */
-#endif /* TIN_LD */
-
-				error_message("Characteristics:");
-				error_message("\t"
-/* TODO: complete list and do some useful grouping */
-#ifdef NNTP_ONLY
-				"+NNTP_ONLY "
-#else
-#	ifdef NNTP_ABLE
-				"+NNTP_ABLE "
-#	else
-				"-NNTP_ABLE "
-#	endif /* NNTP_ABLE */
-#endif /* NNTP_ONLY */
-#ifdef NO_POSTING
-				"+NO_POSTING "
-#else
-				"-NO_POSTING "
-#endif /* NO_POSTING */
-#ifdef BROKEN_LISTGROUP
-				"+BROKEN_LISTGROUP "
-#else
-				"-BROKEN_LISTGROUP "
-#endif /* BROKEN_LISTGROUP */
-#ifdef XHDR_XREF
-				"+XHDR_XREF"
-#else
-				"-XHDR_XREF"
-#endif /* XHDR_XREF */
-				"\n\t"
-#ifdef HAVE_FASCIST_NEWSADMIN
-				"+HAVE_FASCIST_NEWSADMIN "
-#else
-				"-HAVE_FASCIST_NEWSADMIN "
-#endif /* HAVE_FASCIST_NEWSADMIN */
-#ifdef ENABLE_IPV6
-				"+ENABLE_IPV6 "
-#else
-				"-ENABLE_IPV6 "
-#endif /* ENABLE_IPV6 */
-#ifdef HAVE_COREFILE
-				"+HAVE_COREFILE"
-#else
-				"-HAVE_COREFILE"
-#endif /* HAVE_COREFILE */
-				"\n\t"
-#ifdef NO_SHELL_ESCAPE
-				"+NO_SHELL_ESCAPE "
-#else
-				"-NO_SHELL_ESCAPE "
-#endif /* NO_SHELL_ESCAPE */
-#ifdef DISABLE_PRINTING
-				"+DISABLE_PRINTING "
-#else
-				"-DISABLE_PRINTING "
-#endif /* DISABLE_PRINTING */
-#ifdef DONT_HAVE_PIPING
-				"+DONT_HAVE_PIPING "
-#else
-				"-DONT_HAVE_PIPING "
-#endif /* DONT_HAVE_PIPING */
-#ifdef NO_ETIQUETTE
-				"+NO_ETIQUETTE"
-#else
-				"-NO_ETIQUETTE"
-#endif /* NO_ETIQUETTE */
-				"\n\t"
-#ifdef HAVE_LONG_FILE_NAMES
-				"+HAVE_LONG_FILE_NAMES "
-#else
-				"-HAVE_LONG_FILE_NAMES "
-#endif /* HAVE_LONG_FILE_NAMES */
-#ifdef APPEND_PID
-				"+APPEND_PID "
-#else
-				"-APPEND_PID "
-#endif /* APPEND_PID */
-#ifdef HAVE_MH_MAIL_HANDLING
-				"+HAVE_MH_MAIL_HANDLING"
-#else
-				"-HAVE_MH_MAIL_HANDLING"
-#endif /* HAVE_MH_MAIL_HANDLING */
-				"\n\t"
-#ifdef HAVE_ISPELL
-				"+HAVE_ISPELL "
-#else
-				"-HAVE_ISPELL "
-#endif /* HAVE_ISPELL */
-#ifdef HAVE_METAMAIL
-				"+HAVE_METAMAIL "
-#else
-				"-HAVE_METAMAIL "
-#endif /* HAVE_METAMAIL */
-#ifdef HAVE_SUM
-				"+HAVE_SUM"
-#else
-				"-HAVE_SUM"
-#endif /* HAVE_SUM */
-				"\n\t"
-#ifdef HAVE_COLOR
-				"+HAVE_COLOR "
-#else
-				"-HAVE_COLOR "
-#endif /* HAVE_COLOR */
-#ifdef HAVE_PGP
-				"+HAVE_PGP "
-#else
-				"-HAVE_PGP "
-#endif /* HAVE_PGP */
-#ifdef HAVE_PGPK
-				"+HAVE_PGPK "
-#else
-				"-HAVE_PGPK "
-#endif /* HAVE_PGPK */
-#ifdef HAVE_GPG
-				"+HAVE_GPG"
-#else
-				"-HAVE_GPG"
-#endif /* HAVE_GPG */
-				"\n\t"
-#ifdef MIME_BREAK_LONG_LINES
-				"+MIME_BREAK_LONG_LINES "
-#else
-				"-MIME_BREAK_LONG_LINES "
-#endif /* MIME_BREAK_LONG_LINES */
-#ifdef MIME_STRICT_CHARSET
-				"+MIME_STRICT_CHARSET "
-#else
-				"-MIME_STRICT_CHARSET "
-#endif /* MIME_STRICT_CHARSET */
-#ifdef CHARSET_CONVERSION
-				"+CHARSET_CONVERSION"
-#else
-				"-CHARSET_CONVERSION"
-#endif /* CHARSET_CONVERSION */
-				"\n\t"
-#ifdef MULTIBYTE_ABLE
-				"+MULTIBYTE_ABLE "
-#else
-				"-MULTIBYTE_ABLE "
-#endif /* MULTIBYTE_ABLE */
-#ifdef NO_LOCALE
-				"+NO_LOCALE"
-#else
-				"-NO_LOCALE"
-#endif /* NO_LOCALE */
-				"\n\t"
-#ifdef USE_CANLOCK
-				"+USE_CANLOCK "
-#else
-				"-USE_CANLOCK "
-#endif /* USE_CANLOCK */
-#ifdef EVIL_INSIDE
-				"+EVIL_INSIDE "
-#else
-				"-EVIL_INSIDE "
-#endif /* EVIL_INSIDE */
-#ifdef FORGERY
-				"+FORGERY "
-#else
-				"-FORGERY "
-#endif /* FORGERY */
-#ifdef TINC_DNS
-				"+TINC_DNS "
-#else
-				"-TINC_DNS "
-#endif /* TINC_DNS */
-#ifdef ENFORCE_RFC1034
-				"+ENFORCE_RFC1034"
-#else
-				"-ENFORCE_RFC1034"
-#endif /* ENFORCE_RFC1034 */
-				);
-				error_message("\t"
-#ifdef REQUIRE_BRACKETS_IN_DOMAIN_LITERAL
-				"+REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
-#else
-				"-REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
-#endif /* REQUIRE_BRACKETS_IN_DOMAIN_LITERAL */
-#ifdef FOLLOW_USEFOR_DRAFT
-				"+FOLLOW_USEFOR_DRAFT"
-#else
-				"-FOLLOW_USEFOR_DRAFT"
-#endif /* FOLLOW_USEFOR_DRAFT */
-				);
+				tin_version_info(stderr);
 				exit(EXIT_SUCCESS);
 				/* keep lint quiet: */
 				/* FALLTHROUGH */
