@@ -3,10 +3,10 @@
  *  Module    : misc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-06-07
+ *  Updated   : 2005-02-12
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,9 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
+#ifndef VERSION_H
+#	include "version.h"
+#endif /* !VERSION_H */
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
@@ -551,7 +554,7 @@ tin_done(
 			group = &active[my_group[i]];
 			if (group->read_during_session) {
 				if (ask) {
-					if (prompt_yn(cLINES, _(txt_catchup_all_read_groups), FALSE) == 1) {
+					if (prompt_yn(_(txt_catchup_all_read_groups), FALSE) == 1) {
 						ask = FALSE;
 						tinrc.thread_articles = THREAD_NONE;	/* speeds up index loading */
 					} else
@@ -568,7 +571,8 @@ tin_done(
 	 * chance to try again
 	 */
 	if (!no_write) {
-		forever {
+		i = 3; /* max retries */
+		while (i--) {
 			if (((wrote_newsrc_lines = write_newsrc()) >= 0L) && (wrote_newsrc_lines >= read_newsrc_lines)) {
 				if (!batch_mode || verbose)
 					my_fputs(_(txt_newsrc_saved), stdout);
@@ -587,7 +591,7 @@ tin_done(
 			}
 
 			if (!batch_mode) {
-				if (!prompt_yn(cLINES, _(txt_newsrc_again), TRUE))
+				if (prompt_yn(_(txt_newsrc_again), TRUE) <= 0)
 					break;
 			}
 		}
@@ -805,16 +809,21 @@ invoke_cmd(
 		need_resize = cYes;		/* Flag a redraw */
 	}
 
-#ifdef VMS
-	success = (ret != 0);
+#ifdef IGNORE_SYSTEM_STATUS
+	return TRUE;
 #else
-	success = (ret == 0);
-#endif /* VMS */
+
+#	ifdef VMS
+		success = (ret != 0);
+#	else
+		success = (ret == 0);
+#	endif /* VMS */
 
 	if (!success || system_status != 0)
 		error_message(_(txt_command_failed), nam);
 
 	return success;
+#endif /* IGNORE_SYSTEM_STATUS */
 }
 
 
@@ -839,7 +848,6 @@ draw_percent_mark(
 	snprintf(buf, sizeof(buf), "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	if ((wbuf = char2wchar_t(buf)) != NULL) {
-		wconvert_to_printable(wbuf);
 		len = wcswidth(wbuf, wcslen(wbuf) + 1);
 		free(wbuf);
 	} else
@@ -904,6 +912,8 @@ dir_name(
 
 /*
  * Return TRUE if new mail has arrived
+ *
+ * TODO: why not cache the mailbox_name?
  */
 t_bool
 mail_check(
@@ -3528,7 +3538,7 @@ strip_line(
 }
 
 
-#if defined(CHARSET_CONVERSION) || defined(HAVE_UNICODE_NORMALIZATION)
+#if defined(CHARSET_CONVERSION) || (defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE))
 /*
  * 'check' a given UTF-8 strig and '?'-out illegal sequences
  * TODO: is this check complete?
@@ -3568,88 +3578,95 @@ utf8_valid(
 			numc++;
 		} while ((d <<= 1) & 0x80);	/* get sequence length */
 
-		d = *c;
-		e = *(c + 1);
-
-		switch (numc) {
-			case 2:
-				/* out of range or sequences which would also fit into 1 byte */
-				if (d < 0xc2 || d > 0xdf)
-					illegal = TRUE;
-				break;
-
-			case 3:
-				f = *(c + 2);
-				/* out of range or sequences which would also fit into 2 bytes */
-				if (d < 0xe0 || d > 0xef || (d == 0xe0 && e < 0xa0))
-					illegal = TRUE;
-				/* U+D800 ... U+DFFF */
-				if (d == 0xed && e > 0x9f)
-					illegal = TRUE;
-				/* U+FDD0 ... U+FDEF */
-				if (d == 0xef && e == 0xb7 && (f >= 0x90 && f <= 0xaf))
-					illegal = TRUE;
-				/* U+FFFE, U+FFFF */
-				if (d == 0xef && e == 0xbf && (f == 0xbe || f == 0xbf))
-					illegal = TRUE;
-				break;
-
-			case 4:
-				f = *(c + 2);
-				g = *(c + 3);
-				/* out of range or sequences which would also fit into 3 bytes */
-				if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
-					illegal = TRUE;
-				/* largest current used sequence */
-				if ((d == 0xf4 && e > 0x8f) || d > 0xf4)
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
-				if (d == 0xf0 && (e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+4FFFE, U+4FFFF, U+5FFFE, U+5FFFF, U+6FFFE, U+6FFFF, U+7FFFE, U+7FFFF */
-				/* U+8FFFE, U+8FFFF, U+9FFFE, U+9FFFF, U+AFFFE, U+AFFFF, U+BFFFE, U+BFFFF */
-				/* U+CFFFE, U+CFFFF, U+DFFFE, U+DFFFF, U+EFFFE, U+EFFFF, U+FFFFE, U+FFFFF */
-				if ((d == 0xf1 || d == 0xf2 || d == 0xf3) && (e == 0x8f || e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+10FFFE, U+10FFFF */
-				if (d == 0xf4 && e == 0x8f && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				break;
-
-#	if 0	/* currently not used, see also check above */
-			case 5:
-				/* out of range or sequences which would also fit into 4 bytes */
-				if (d < 0xf8 || d > 0xfb || (d == 0xf8 && e < 0x88))
-					illegal = TRUE;
-				break;
-
-			case 6:
-				/* out of range or sequences which would also fit into 5 bytes */
-				if (d < 0xfc || d > 0xfd || (d == 0xfc && e < 0x84))
-					illegal = TRUE;
-				break;
-#	endif /* 0 */
-
-			default:
-				/*
-				 * with the check for plain US-ASCII above all other sequence
-				 * length are illegal.
-				 */
+		d = 1;
+		while (d < numc) {
+			if (*(c + d) == '\0' || *(c + d) == '\n')
 				illegal = TRUE;
-				break;
+			d--;
 		}
 
 		if (!illegal) {
-			for (d = 1; d < numc; d++) {
-				e = *(c + d);
-				if (e < 0x80 || e > 0xbf || *(c + d) == '\0' || *(c + d) == '\n')
+			d = *c;
+			e = *(c + 1);
+
+			switch (numc) {
+				case 2:
+					/* out of range or sequences which would also fit into 1 byte */
+					if (d < 0xc2 || d > 0xdf)
+						illegal = TRUE;
+					break;
+
+				case 3:
+					f = *(c + 2);
+					/* out of range or sequences which would also fit into 2 bytes */
+					if (d < 0xe0 || d > 0xef || (d == 0xe0 && e < 0xa0))
+						illegal = TRUE;
+					/* U+D800 ... U+DFFF */
+					if (d == 0xed && e > 0x9f)
+						illegal = TRUE;
+					/* U+FDD0 ... U+FDEF */
+					if (d == 0xef && e == 0xb7 && (f >= 0x90 && f <= 0xaf))
+						illegal = TRUE;
+					/* U+FFFE, U+FFFF */
+					if (d == 0xef && e == 0xbf && (f == 0xbe || f == 0xbf))
+						illegal = TRUE;
+					break;
+
+				case 4:
+					f = *(c + 2);
+					g = *(c + 3);
+					/* out of range or sequences which would also fit into 3 bytes */
+					if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
+						illegal = TRUE;
+					/* largest current used sequence */
+					if ((d == 0xf4 && e > 0x8f) || d > 0xf4)
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
+					if (d == 0xf0 && (e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+4FFFE, U+4FFFF, U+5FFFE, U+5FFFF, U+6FFFE, U+6FFFF, U+7FFFE, U+7FFFF */
+					/* U+8FFFE, U+8FFFF, U+9FFFE, U+9FFFF, U+AFFFE, U+AFFFF, U+BFFFE, U+BFFFF */
+					/* U+CFFFE, U+CFFFF, U+DFFFE, U+DFFFF, U+EFFFE, U+EFFFF, U+FFFFE, U+FFFFF */
+					if ((d == 0xf1 || d == 0xf2 || d == 0xf3) && (e == 0x8f || e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+10FFFE, U+10FFFF */
+					if (d == 0xf4 && e == 0x8f && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					break;
+
+#	if 0	/* currently not used, see also check above */
+				case 5:
+					/* out of range or sequences which would also fit into 4 bytes */
+					if (d < 0xf8 || d > 0xfb || (d == 0xf8 && e < 0x88))
+						illegal = TRUE;
+					break;
+
+				case 6:
+					/* out of range or sequences which would also fit into 5 bytes */
+					if (d < 0xfc || d > 0xfd || (d == 0xfc && e < 0x84))
+						illegal = TRUE;
+					break;
+#	endif /* 0 */
+
+				default:
+					/*
+					 * with the check for plain US-ASCII above all other sequence
+					 * length are illegal.
+					 */
 					illegal = TRUE;
+					break;
 			}
 		}
 
+		for (d = 1; d < numc && !illegal; d++) {
+			e = *(c + d);
+			if (e < 0x80 || e > 0xbf || e == '\0' || e == '\n')
+				illegal = TRUE;
+		}
+
 		if (!illegal)
 			c += numc; /* skip over valid sequence */
 		else {
@@ -3664,7 +3681,7 @@ utf8_valid(
 	}
 	return line;
 }
-#endif /* CHARSET_CONVERSION || HAVE_UNICODE_NORMALIZATION */
+#endif /* CHARSET_CONVERSION || (MULTIBYTE_ABLE && !NO_LOCALE) */
 
 
 char *idna_decode(
@@ -3694,6 +3711,261 @@ char *idna_decode(
 }
 
 
+int
+tin_version_info(
+	FILE *fp)
+{
+	int wlines = 0;	/* written lines */
+
+#if defined(__DATE__) && defined(__TIME__)
+	fprintf(fp, _("Version: %s %s release %s (\"%s\") %s %s\n"),
+		PRODUCT, VERSION, RELEASEDATE, RELEASENAME, __DATE__, __TIME__);
+#else
+	fprintf(fp, _("Version: %s %s release %s (\"%s\")\n"),
+	       PRODUCT, VERSION, RELEASEDATE, RELEASENAME);
+#endif /* __DATE__ && __TIME__ */
+	wlines++;
+
+#ifdef SYSTEM_NAME
+	fprintf(fp, "Platform:\n");
+	fprintf(fp, "\tOS-Name  = \"%s\"\n", SYSTEM_NAME);
+	wlines += 2;
+#endif /* SYSTEM_NAME */
+
+#ifdef TIN_CC
+	fprintf(fp, "Compiler:\n");
+	fprintf(fp, "\tCC       = \"%s\"\n", TIN_CC);
+	wlines += 2;
+#	ifdef TIN_CFLAGS
+		fprintf(fp, "\tCFLAGS   = \"%s\"\n", TIN_CFLAGS);
+		wlines++;
+#	endif /* TIN_CFLAGS */
+#	ifdef TIN_CPP
+		fprintf(fp, "\tCPP      = \"%s\"\n", TIN_CPP);
+		wlines++;
+#	endif /* TIN_CPP */
+#	ifdef TIN_CPPFLAGS
+		fprintf(fp, "\tCPPFLAGS = \"%s\"\n", TIN_CPPFLAGS);
+		wlines++;
+#	endif /* TIN_CPPFLAGS */
+#endif /* TIN_CC */
+
+#ifdef TIN_LD
+	fprintf(fp, "Linker and Libraries:\n");
+	fprintf(fp, "\tLD       = \"%s\"\n", TIN_LD);
+	wlines += 2;
+#	ifdef TIN_LDFLAGS
+		fprintf(fp, "\tLDFLAGS  = \"%s\"\n", TIN_LDFLAGS);
+		wlines++;
+#	endif /* TIN_LDFLAGS */
+#	ifdef TIN_LIBS
+		fprintf(fp, "\tLIBS     = \"%s\"\n", TIN_LIBS);
+		wlines++;
+#	endif /* TIN_LIBS */
+#endif /* TIN_LD */
+	fprintf(fp, "\tPCRE     = \"%s\"\n", pcre_version());
+	wlines++;
+
+	fprintf(fp, "Characteristics:\n\t"
+/* TODO: complete list and do some useful grouping */
+#ifdef NNTP_ONLY
+			"+NNTP_ONLY "
+#else
+#	ifdef NNTP_ABLE
+			"+NNTP_ABLE "
+#	else
+			"-NNTP_ABLE "
+#	endif /* NNTP_ABLE */
+#endif /* NNTP_ONLY */
+#ifdef NO_POSTING
+			"+NO_POSTING "
+#else
+			"-NO_POSTING "
+#endif /* NO_POSTING */
+#ifdef BROKEN_LISTGROUP
+			"+BROKEN_LISTGROUP "
+#else
+			"-BROKEN_LISTGROUP "
+#endif /* BROKEN_LISTGROUP */
+#ifdef XHDR_XREF
+			"+XHDR_XREF"
+#else
+			"-XHDR_XREF"
+#endif /* XHDR_XREF */
+			"\n\t"
+#ifdef HAVE_FASCIST_NEWSADMIN
+			"+HAVE_FASCIST_NEWSADMIN "
+#else
+			"-HAVE_FASCIST_NEWSADMIN "
+#endif /* HAVE_FASCIST_NEWSADMIN */
+#ifdef ENABLE_IPV6
+			"+ENABLE_IPV6 "
+#else
+			"-ENABLE_IPV6 "
+#endif /* ENABLE_IPV6 */
+#ifdef HAVE_COREFILE
+			"+HAVE_COREFILE"
+#else
+			"-HAVE_COREFILE"
+#endif /* HAVE_COREFILE */
+			"\n\t"
+#ifdef NO_SHELL_ESCAPE
+			"+NO_SHELL_ESCAPE "
+#else
+			"-NO_SHELL_ESCAPE "
+#endif /* NO_SHELL_ESCAPE */
+#ifdef DISABLE_PRINTING
+			"+DISABLE_PRINTING "
+#else
+			"-DISABLE_PRINTING "
+#endif /* DISABLE_PRINTING */
+#ifdef DONT_HAVE_PIPING
+			"+DONT_HAVE_PIPING "
+#else
+			"-DONT_HAVE_PIPING "
+#endif /* DONT_HAVE_PIPING */
+#ifdef NO_ETIQUETTE
+			"+NO_ETIQUETTE"
+#else
+			"-NO_ETIQUETTE"
+#endif /* NO_ETIQUETTE */
+			"\n\t"
+#ifdef HAVE_LONG_FILE_NAMES
+			"+HAVE_LONG_FILE_NAMES "
+#else
+			"-HAVE_LONG_FILE_NAMES "
+#endif /* HAVE_LONG_FILE_NAMES */
+#ifdef APPEND_PID
+			"+APPEND_PID "
+#else
+			"-APPEND_PID "
+#endif /* APPEND_PID */
+#ifdef HAVE_MH_MAIL_HANDLING
+			"+HAVE_MH_MAIL_HANDLING"
+#else
+			"-HAVE_MH_MAIL_HANDLING"
+#endif /* HAVE_MH_MAIL_HANDLING */
+			"\n\t"
+#ifdef HAVE_ISPELL
+			"+HAVE_ISPELL "
+#else
+			"-HAVE_ISPELL "
+#endif /* HAVE_ISPELL */
+#ifdef HAVE_METAMAIL
+			"+HAVE_METAMAIL "
+#else
+			"-HAVE_METAMAIL "
+#endif /* HAVE_METAMAIL */
+#ifdef HAVE_SUM
+			"+HAVE_SUM"
+#else
+			"-HAVE_SUM"
+#endif /* HAVE_SUM */
+			"\n\t"
+#ifdef HAVE_COLOR
+			"+HAVE_COLOR "
+#else
+			"-HAVE_COLOR "
+#endif /* HAVE_COLOR */
+#ifdef HAVE_PGP
+			"+HAVE_PGP "
+#else
+			"-HAVE_PGP "
+#endif /* HAVE_PGP */
+#ifdef HAVE_PGPK
+			"+HAVE_PGPK "
+#else
+			"-HAVE_PGPK "
+#endif /* HAVE_PGPK */
+#ifdef HAVE_GPG
+			"+HAVE_GPG"
+#else
+			"-HAVE_GPG"
+#endif /* HAVE_GPG */
+			"\n\t"
+#ifdef MIME_BREAK_LONG_LINES
+			"+MIME_BREAK_LONG_LINES "
+#else
+			"-MIME_BREAK_LONG_LINES "
+#endif /* MIME_BREAK_LONG_LINES */
+#ifdef MIME_STRICT_CHARSET
+			"+MIME_STRICT_CHARSET "
+#else
+			"-MIME_STRICT_CHARSET "
+#endif /* MIME_STRICT_CHARSET */
+#ifdef CHARSET_CONVERSION
+			"+CHARSET_CONVERSION"
+#else
+			"-CHARSET_CONVERSION"
+#endif /* CHARSET_CONVERSION */
+			"\n\t"
+#ifdef MULTIBYTE_ABLE
+			"+MULTIBYTE_ABLE "
+#else
+			"-MULTIBYTE_ABLE "
+#endif /* MULTIBYTE_ABLE */
+#ifdef NO_LOCALE
+			"+NO_LOCALE"
+#else
+			"-NO_LOCALE"
+#endif /* NO_LOCALE */
+			"\n\t"
+#ifdef USE_CANLOCK
+			"+USE_CANLOCK "
+#else
+			"-USE_CANLOCK "
+#endif /* USE_CANLOCK */
+#ifdef EVIL_INSIDE
+			"+EVIL_INSIDE "
+#else
+			"-EVIL_INSIDE "
+#endif /* EVIL_INSIDE */
+#ifdef FORGERY
+			"+FORGERY "
+#else
+			"-FORGERY "
+#endif /* FORGERY */
+#ifdef TINC_DNS
+			"+TINC_DNS "
+#else
+			"-TINC_DNS "
+#endif /* TINC_DNS */
+#ifdef ENFORCE_RFC1034
+			"+ENFORCE_RFC1034"
+#else
+			"-ENFORCE_RFC1034"
+#endif /* ENFORCE_RFC1034 */
+			"\n\t"
+#ifdef REQUIRE_BRACKETS_IN_DOMAIN_LITERAL
+			"+REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
+#else
+			"-REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
+#endif /* REQUIRE_BRACKETS_IN_DOMAIN_LITERAL */
+#ifdef FOLLOW_USEFOR_DRAFT
+			"+FOLLOW_USEFOR_DRAFT"
+#else
+			"-FOLLOW_USEFOR_DRAFT"
+#endif /* FOLLOW_USEFOR_DRAFT */
+			"\n");
+	wlines += 11;
+
+	fflush(fp);
+	return wlines;
+}
+
+
+void
+draw_mark_selected(
+	int i)
+{
+	MoveCursor(INDEX2LNUM(i), MARK_OFFSET);
+	StartInverse();	/* ToggleInverse() doesn't work correct with ncurses4.x */
+	my_fputc(tinrc.art_marked_selected, stdout);
+	EndInverse();	/* ToggleInverse() doesn't work correct with ncurses4.x */
+	return;
+}
+
+
 #if 0
 /*
  * Stat a mail/news article to see if it still exists
