@@ -3,10 +3,10 @@
  *  Module    : my_tmpfile.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 2001-03-11
- *  Updated   : 2001-04-23
+ *  Updated   : 2004-06-30
  *  Notes     :
  *
- * Copyright (c) 2001-2004 Urs Janssen <urs@tin.org>
+ * Copyright (c) 2001-2005 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,8 @@ my_tmpfile(
 
 	if (filename != NULL && name_size > 0) {
 		if (!need_name) {
-			FILE *fp = (FILE *) 0;
+			FILE *fp;
+
 			if ((fp = tmpfile()) != NULL)
 				fd = fileno(fp);
 #ifdef DEBUG
