@@ -3,10 +3,10 @@
  *  Module    : prompt.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2004-06-12
+ *  Updated   : 2005-02-12
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,9 +41,9 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
 
 static char *prompt_slk_message;	/* prompt message for prompt_slk_redraw */
 
@@ -68,9 +68,7 @@ prompt_num(
 	int num;
 
 	clear_message();
-
 	snprintf(tmp, sizeof(tmp), "%c", ch);
-
 	if ((p = tin_getline(prompt, TRUE, tmp, 0, FALSE, HIST_OTHER)) != NULL) {
 		STRCPY(tmp, p);
 		num = atoi(tmp);
@@ -78,7 +76,6 @@ prompt_num(
 		num = -1;
 
 	clear_message();
-
 	return num;
 }
 
@@ -115,16 +112,13 @@ prompt_default_string(
 	char *p;
 
 	clear_message();
-
 	if ((p = tin_getline(prompt, FALSE, default_prompt, buf_len, FALSE, which_hist)) == NULL) {
 		buf[0] = '\0';
 		clear_message();
 		return FALSE;
 	}
 	strcpy(buf, p);
-
 	clear_message();
-
 	return TRUE;
 }
 
@@ -148,50 +142,49 @@ prompt_menu_string(
 	 * would lead to a 'n' answer to the reconnect prompt
 	 */
 	fflush(stdin);
-
 	MoveCursor(line, 0);
-
 	if ((p = tin_getline(prompt, FALSE, var, 0, FALSE, HIST_OTHER)) == NULL)
 		return FALSE;
 
 	strcpy(var, p);
-
 	return TRUE;
 }
 
 
 /*
  * prompt_yn
- * prompt user for 'y'es or 'n'o decision. "prompt" will be displayed in line
- * "line" giving the default answer "default_answer".
- * TODO: 'line' is constant - can we remove it ?
+ * prompt user for 'y'es or 'n'o decision. "prompt" will be displayed in the
+ * last line giving the default answer "default_answer".
  * The function returns 1 if the user decided "yes", -1 if the user wanted
- * to escape, or 0 for any other key or decision.
+ * to escape, or 0 for any other decision.
  */
 int
 prompt_yn(
-	int line,
 	const char *prompt,
 	t_bool default_answer)
 {
 	char *keyprompt;
 	char keyno[MAXKEYLEN], keyyes[MAXKEYLEN];
-	int ch = 'y', prompt_ch = 'y'; /* why not iKeyPromptYes? */
+	char yes, no, prompt_ch;
+	int ch;
 	size_t maxlen;
-	t_bool yn_loop = TRUE;
+	t_function func;
 
 /*	fflush(stdin); */		/* Prevent finger trouble from making important decisions */
 
-	(void) printascii(keyyes, (default_answer ? toupper(map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)));
-	(void) printascii(keyno, (!default_answer ? toupper(map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)));
+	yes = func_to_key(PROMPT_YES, prompt_keys);
+	no = func_to_key(PROMPT_NO, prompt_keys);
+
+	printascii(keyyes, (default_answer ? toupper(yes) : yes));
+	printascii(keyno, (!default_answer ? toupper(no) : no));
 	maxlen = MAX(strlen(keyyes), strlen(keyno));
 
-	while (yn_loop) {
-		prompt_ch = map_to_local((default_answer ? iKeyPromptYes : iKeyPromptNo), &menukeymap.prompt_yn);
+	do {
+		prompt_ch = (default_answer ? yes : no);
 		keyprompt = (default_answer ? keyyes : keyno);
 
 		if (!cmd_line) {
-			MoveCursor(line, 0);
+			MoveCursor(cLINES, 0);
 			CleartoEOLN();
 		}
 		my_printf("%s (%s/%s) %-*s", prompt, keyyes, keyno, (int) maxlen, keyprompt);
@@ -199,33 +192,31 @@ prompt_yn(
 			cursoron();
 		my_flush();
 		if (!cmd_line)
-			MoveCursor(line, (int) strlen(prompt) + strlen(keyyes) + strlen(keyno) + 5);
+			MoveCursor(cLINES, (int) strlen(prompt) + strlen(keyyes) + strlen(keyno) + 5);
 
 		if (((ch = (char) ReadCh()) == '\n') || (ch == '\r'))
 			ch = prompt_ch;
 
-		yn_loop = FALSE; /* normal case: leave loop */
-
 		switch (ch) {
 			case ESC:	/* (ESC) common arrow keys */
 #	ifdef HAVE_KEY_PREFIX
 			case KEY_PREFIX:
 #	endif /* HAVE_KEY_PREFIX */
 				switch (get_arrow_key(ch)) {
-
 					case KEYMAP_UP:
 					case KEYMAP_DOWN:
 						default_answer = bool_not(default_answer);
-						yn_loop = TRUE; /* don't leave loop */
+						ch = '\0';	/* set to a not bindable key to not leave the loop yet */
 						break;
 
 					case KEYMAP_LEFT:
-						ch = iKeyAbort;
+						ch = ESC;
 						break;
 
 					case KEYMAP_RIGHT:
 						ch = prompt_ch;
 						break;
+
 					default:
 						break;
 				}
@@ -234,20 +225,14 @@ prompt_yn(
 			default:
 				break;
 		}
-	}
+		func = key_to_func(ch, prompt_keys);
+	} while (func == NOT_ASSIGNED);
 
 	if (!cmd_line) {
-		if (line == cLINES)
-			clear_message();
-		else {
-			MoveCursor(line, (int) strlen(prompt));
-			my_fputc(((ch == iKeyAbort) ? prompt_ch : ch), stdout);
-		}
-		cursoroff();
+		clear_message();
 		my_flush();
 	}
-
-	return (tolower((unsigned char) map_to_default(ch, &menukeymap.prompt_yn)) == tolower((unsigned char)iKeyPromptYes)) ? 1 : (ch == iKeyAbort) ? -1 : 0;
+	return (func == PROMPT_YES) ? 1 : (func == GLOBAL_ABORT) ? -1 : 0;
 }
 
 
@@ -271,8 +256,12 @@ prompt_list(
 {
 	int ch, var_orig;
 	int i, offset;
+	int change;
 	int adjust = (strcasecmp(_(list[0]), _(txt_default)) == 0);
 	size_t width = 0;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wbuf;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	var += adjust;
 	size += adjust;
@@ -287,31 +276,68 @@ prompt_list(
 	show_menu_help(help_text);
 	cursoron();
 
-	do {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		wchar_t *wbuf;
-
-		if ((wbuf = char2wchar_t(_(prompt_text))) != NULL) {
-			wconvert_to_printable(wbuf);
-			offset = wcswidth(wbuf, wcslen(wbuf) + 1);
-			if (offset == -1) {
-				/* something went wrong, use wcslen as fallback */
-				offset = (int) wcslen(wbuf);
-			}
-
-			free(wbuf);
-		} else
+	if ((wbuf = char2wchar_t(_(prompt_text))) != NULL) {
+		if ((offset = wcswidth(wbuf, wcslen(wbuf) + 1)) == -1) /* something went wrong, use wcslen as fallback */
+			offset = (int) wcslen(wbuf);
+		free(wbuf);
+	} else
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-			offset = (int) strlen(_(prompt_text));
+		offset = (int) strlen(_(prompt_text));
 
+	do {
 		MoveCursor(row, col + offset);
-		if ((ch = (char) ReadCh()) == ' ') {
+		ch = (char) ReadCh();
 
+		/*
+		 * change:
+		 *   1 = move to the next list element
+		 *   0 = do nothing
+		 *  -1 = move to the previous list element
+		 *
+		 *  if an arrow key was pressed change ch to an other value
+		 *  otherwise we will exit the while loop
+		 */
+		switch (ch) {
+			case ' ':
+				change = 1;
+				break;
+
+			case ESC:	/* (ESC) common arrow keys */
+#	ifdef HAVE_KEY_PREFIX
+			case KEY_PREFIX:
+#	endif /* HAVE_KEY_PREFIX */
+				switch (get_arrow_key(ch)) {
+					case KEYMAP_UP:
+						change = -1;
+						ch = ' ';
+						break;
+
+					case KEYMAP_DOWN:
+						change = 1;
+						ch = ' ';
+						break;
+
+					default:
+						change = 0;
+						break;
+				}
+				break;
+
+			default:
+				change = 0;
+				break;
+		}
+
+		if (change) {
 			/*
-			 * Increment list, looping around at the max
+			 * increment or decrement list, loop around at the limits
 			 */
-			++var;
-			var %= size;
+			var += change;
+			if (var < 0)
+				var = size - 1;
+			else
+				var %= size;
 
 			my_printf("%-*s", (int) width, _(list[var]));
 			my_flush();
@@ -320,13 +346,11 @@ prompt_list(
 
 	if (ch == ESC) {
 		var = var_orig;
-
 		my_printf("%-*s", (int) width, _(list[var]));
 		my_flush();
 	}
 
 	cursoroff();
-
 	return (var - adjust);
 }
 
@@ -415,9 +439,7 @@ prompt_option_num(
 	STRCPY(number, p);
 	num = atoi(number);
 	*(option_table[option].variable) = num;
-
 	clear_message();
-
 	return TRUE;
 }
 
@@ -455,7 +477,6 @@ prompt_option_char(
 	} while (!*p);
 
 	*variable = p[0];
-
 	clear_message();
 	return TRUE;
 }
@@ -566,7 +587,6 @@ sized_message(
 	wchar_t *wformat;
 
 	if ((wformat = char2wchar_t(format)) != NULL) {
-		wconvert_to_printable(wformat);
 		/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
 		max_len = cCOLS - wcswidth(wformat, wcslen(wformat) + 1) + 2 - 1;
 		free(wformat);
@@ -574,8 +594,7 @@ sized_message(
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		max_len = cCOLS - strlen(format) + 2 - 1;	/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
 
-	buf = my_malloc(strlen(subject) + 1);
-	strunc(subject, buf, strlen(subject) + 1, max_len);
+	buf = strunc(subject, max_len);
 
 	*result = fmt_string(format, buf);
 	free(buf);
@@ -588,24 +607,24 @@ sized_message(
  * Implement the Single-Letter-Key mini menus at the bottom of the screen
  * eg, Press a)ppend, o)verwrite, q)uit :
  */
-int
+t_function
 prompt_slk_response(
-	int ch_default,
-	const t_menukeys /* char */ *responses,
+	t_function default_func,
+	const struct keylist keys,
 	const char *fmt,
 	...)
 {
 	va_list ap;
 	char ch;
 	char buf[LEN];
+	t_function func;
 
 	va_start(ap, fmt);
 	vsnprintf(buf, sizeof(buf), fmt, ap);
 	va_end(ap);
 
 	prompt_slk_message = my_malloc(strlen(buf) + 2);
-	ch_default = map_to_local(ch_default, responses);
-	snprintf(prompt_slk_message, strlen(buf) + 2, "%s%c", buf, ch_default);
+	snprintf(prompt_slk_message, strlen(buf) + 2, "%s%c", buf, func_to_key(default_func, keys));
 
 	input_context = cPromptSLK;
 
@@ -613,7 +632,9 @@ prompt_slk_response(
 		prompt_slk_redraw();		/* draw the prompt */
 
 		if ((ch = ReadCh()) == '\r' || ch == '\n')
-			ch = ch_default;
+			func = default_func;
+		else
+			func = key_to_func(ch, keys);
 
 		/*
 		 * TODO: ignore special-keys which are represented as a
@@ -639,14 +660,13 @@ prompt_slk_response(
 			}
 		}
 #endif /* 0 */
-
-	} while (!strchr(responses->localkeys, ch));
+	} while (func == NOT_ASSIGNED);
 
 	input_context = cNone;
 	FreeAndNull(prompt_slk_message);
 
 	clear_message();
-	return map_to_default(ch, responses);
+	return func;
 }
 
 
@@ -665,7 +685,6 @@ prompt_slk_redraw(
 	/* get the cursor _just_ right */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	if ((wtmp = char2wchar_t(prompt_slk_message)) != NULL) {
-		wconvert_to_printable(wtmp);
 		column = wcswidth(wtmp, wcslen(wtmp) + 1) - 1;
 		free(wtmp);
 	} else
@@ -699,6 +718,7 @@ prompt_continue(
 #	endif /* HAVE_KEY_PREFIX */
 			(void) get_arrow_key(ch);
 			/* FALLTHROUGH */
+
 		default:
 			break;
 	}
