@@ -3,10 +3,10 @@
  *  Module    : rfc1524.c
  *  Author    : Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-05-15
- *  Updated   : 2002-08-24
+ *  Updated   : 2004-11-28
  *  Notes     : mailcap parsing as defined in RFC 1524
  *
- * Copyright (c) 2000-2004 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2005 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -70,19 +70,18 @@ get_mailcap_entry(
 	char buf[LEN];
 	char filename[LEN];	/* name of current mailcap file */
 	char mailcap[LEN];	/* full match */
-	char mailcaps[LEN];	/* possible mailcap files */
+	char *mailcaps = NULL;	/* possible mailcap files */
 	char wildcap[LEN];	/* basetype match */
 	t_mailcap *foo = (t_mailcap *) 0;
 
-	mailcaps[0] = '\0';
 	/* build list of mailcap files */
-	if ((ptr = getenv("MAILCAPS")) != NULL) {
-		if (strlen(ptr)) {
-			STRCPY(mailcaps, ptr);
-			strncat(mailcaps, ":", sizeof(mailcaps) - 1);
-		}
-	}
-	strncat(mailcaps, DEFAULT_MAILCAPS, sizeof(mailcaps) - 1);
+	if ((ptr = getenv("MAILCAPS")) != NULL && strlen(ptr))
+			mailcaps = my_strdup(ptr);
+	if (mailcaps != NULL) {
+		mailcaps = my_realloc(mailcaps, strlen(mailcaps) + strlen(DEFAULT_MAILCAPS) + 2);
+		strcat(strcat(mailcaps, ":"), DEFAULT_MAILCAPS);
+	} else
+		mailcaps = my_strdup(DEFAULT_MAILCAPS);
 
 	mailcap[0] = '\0';
 	wildcap[0] = '\0';
@@ -117,6 +116,7 @@ get_mailcap_entry(
 								foo = parse_mailcap_line(mailcap, part, path);
 								if (foo != NULL) {
 									fclose(fp); /* perfect match with test succeded (if given) */
+									free(mailcaps);
 									return foo;
 								}
 							} else {
@@ -133,6 +133,7 @@ get_mailcap_entry(
 					} /* else invalid mailcap line (no /), no action required */
 					if (strlen(wildcap)) {	/* we just had a wildmat match */
 						fclose(fp);
+						free(mailcaps);
 						return foo;
 					}
 				} /* while ((fgets(ptr, ... */
@@ -141,6 +142,7 @@ get_mailcap_entry(
 		} /* else strfpath() failed, no action required */
 		nptr = strtok(NULL, ":"); /* get next filename */
 	}
+	free(mailcaps);
 	foo = (t_mailcap *) 0; /* no match, weed out possible junk */
 	return foo;
 }
@@ -161,19 +163,19 @@ parse_mailcap_line(
 
 	/* malloc and init */
 	tmailcap = my_malloc(sizeof(t_mailcap));
-	tmailcap->type = (char *) 0;
-	tmailcap->command = (char *) 0;
+	tmailcap->type = NULL;
+	tmailcap->command = NULL;
 	tmailcap->needsterminal = FALSE;
 	tmailcap->copiousoutput = FALSE;
 	tmailcap->textualnewlines = 0;
-	tmailcap->description = (char *) 0;
-	tmailcap->test = (char *) 0;
-	tmailcap->nametemplate = (char *) 0;
-	tmailcap->compose = (char *) 0;
-	tmailcap->composetyped = (char *) 0;
-	tmailcap->edit = (char *) 0;
-	tmailcap->print = (char *) 0;
-	tmailcap->x11bitmap = (char *) 0;
+	tmailcap->description = NULL;
+	tmailcap->test = NULL;
+	tmailcap->nametemplate = NULL;
+	tmailcap->compose = NULL;
+	tmailcap->composetyped = NULL;
+	tmailcap->edit = NULL;
+	tmailcap->print = NULL;
+	tmailcap->x11bitmap = NULL;
 
 	optr = ptr = my_strdup(mailcap);
 
@@ -353,7 +355,7 @@ expand_mailcap_meta(
 	int quote = no_quote;
 	size_t linelen, space, olen;
 
-	if ((ptr = strchr(mailcap, '%')) == NULL) /* nothing to expand */
+	if (!(strchr(mailcap, '%'))) /* nothing to expand */
 		return my_strdup(mailcap); /* waste of mem, but simplyfies the frees */
 
 	linelen = LEN * 2;					/* initial maxlen */
