@@ -3,10 +3,10 @@
  *  Module    : rfc2047.c
  *  Author    : Chris Blum <chris@resolution.de>
  *  Created   : 1995-09-01
- *  Updated   : 2004-03-10
+ *  Updated   : 2004-11-26
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2004 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2005 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -72,7 +72,7 @@ static int base64_rank_table_built;
 static t_bool quoteflag;
 
 /* fixed prefix and default part for tin-generated MIME boundaries */
-static const char MIME_BOUNDARY_PREFIX[]       = "=_tin=_";
+static const char MIME_BOUNDARY_PREFIX[] = "=_tin=_";
 static const char MIME_BOUNDARY_DEFAULT_PART[] = "====____====____====____";
 /* required size of a buffer containing a MIME boundary, including the final '\0' */
 enum {
@@ -279,7 +279,7 @@ rfc1522_decode(
 				c++;
 				encoding = tolower((unsigned char)*c);
 				if (encoding == 'b')
-					(void) mmdecode((char *) 0, 'b', 0, (char *) 0);	/* flush */
+					(void) mmdecode(NULL, 'b', 0, NULL);	/* flush */
 				c++;
 				if (*c == '?') {
 					c++;
@@ -789,7 +789,7 @@ do_rfc15211522_encode(
 	BodyPtr body_encode;
 	int i;
 #ifdef CHARSET_CONVERSION
-	int mmnwcharset = 0;
+	int mmnwcharset;
 
 	if (group) /* Posting */
 		mmnwcharset = group->attribute->mm_network_charset;
@@ -843,12 +843,12 @@ do_rfc15211522_encode(
 			}
 		}
 	}
-	/*
-	 * This should work as g should be at least as long as f.
-	 * We could use ftruncate(), but this isn't part of ISO-C.
-	 */
-	rewind(f);
+
 	rewind(g);
+	rewind(f);
+#ifdef HAVE_FTRUNCATE
+	ftruncate(fileno(f), 0);
+#endif /* HAVE_FTRUNCATE */
 
 	/* copy header */
 	while (fgets(buffer, 2048, g) && !isreturn(buffer[0]))
