@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2004-06-07
+ *  Updated   : 2005-03-08
  *  Notes     :
  *
- * Copyright (c) 1997-2004 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2005 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -268,7 +268,7 @@ strpbrk(
 				return ptr1;
 		}
 	}
-	return (char *) 0;
+	return NULL;
 }
 #endif /* !HAVE_STRPBRK */
 
@@ -466,6 +466,9 @@ str_trim(
 	char *wp;		/* writing string pointer */
 	char *ls;		/* last space */
 
+	if (string == NULL)
+		return NULL;
+
 	for (rp = wp = ls = string; isspace((int) *rp); rp++)		/* Skip leading space */
 		;
 
@@ -714,40 +717,44 @@ wchar_t2char(
 
 
 /*
- * copy wide-chars from '*from' to '*to' until 'columns' columns are filled
- * pad with spaces if necessary
+ * returns a new string fitting into 'columns' columns
+ * if pad is TRUE the resulting string will be filled with spaces if necessary
  */
-void
+wchar_t *
 wcspart(
-	wchar_t *to,
-	const wchar_t *from,
+	const wchar_t *wstr,
 	int columns,
-	int size_to,
 	t_bool pad)
 {
-	int n, i = 0;
+	int used = 0;
 	wchar_t *ptr, *wbuf;
 
-	/* make sure all characters in from are printable */
-	wbuf = my_wcsdup(from);
+	wbuf = my_wcsdup(wstr);
+	/* make sure all characters in wbuf are printable */
 	ptr = wconvert_to_printable(wbuf);
 
-	to[0] = (wint_t) '\0';
-	while (*ptr && i < size_to && wcswidth(to, size_to - 1) + wcwidth(*ptr) <= columns) {
-		to[i] = *ptr;
-		ptr++;
-		to[++i] = (wint_t) '\0';
-	}
+	/* terminate wbuf after 'columns' columns */
+	while (*ptr && used + wcwidth(*ptr) <= columns)
+		used += wcwidth(*ptr++);
+	*ptr = (wchar_t) '\0';
 
 	/* pad with spaces */
 	if (pad) {
-		n = columns - wcswidth(to, size_to - 1) + (int) wcslen(to);
-		for (; i < MIN(n, size_to - 1); i++)
-			to[i] = (wint_t) ' ';
-		to[i] = (wint_t) '\0';
-	}
+		int gap;
 
-	free(wbuf);
+		gap = columns - wcswidth(wbuf, wcslen(wbuf) + 1);
+		assert(gap >= 0);
+		wbuf = my_realloc(wbuf, sizeof(wchar_t) * (wcslen(wbuf) + gap + 1));
+		ptr = wbuf + wcslen(wbuf); /* set ptr again to end of wbuf */
+
+		while (gap-- > 0)
+			*ptr++ = (wchar_t) ' ';
+
+		*ptr = (wchar_t) '\0';
+	} else
+		wbuf = my_realloc(wbuf, sizeof(wchar_t) * (wcslen(wbuf) + 1));
+
+	return wbuf;
 }
 #endif /* MULTIBYTE_ABLE && !NOLOCALE */
 
@@ -762,42 +769,34 @@ wcspart(
 char *
 strunc(
 	const char *message,
-	char *buf,
-	size_t buf_len,
 	int len)
 {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	char *tmp;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	wchar_t *wmessage, *wbuf;
-	size_t wbuf_len;
 
 	if ((wmessage = char2wchar_t(message)) != NULL) {
-		wbuf_len = wcslen(wmessage) + 1;
-		wbuf = my_malloc(sizeof(wchar_t) * wbuf_len);
-
-		wstrunc(wmessage, wbuf, wbuf_len, len);
+		wbuf = wstrunc(wmessage, len);
 		free(wmessage);
 
 		if ((tmp = wchar_t2char(wbuf)) != NULL) {
-			strncpy(buf, tmp, buf_len);
-			buf[buf_len - 1] = '\0';
-
 			free(wbuf);
-			free(tmp);
 
-			return buf;
+			return tmp;
 		}
 		free(wbuf);
 	}
 	/* something went wrong using wide-chars, default back to normal chars */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-	if ((int) strlen(message) <= len && len < (int) buf_len)
-		strcpy(buf, message);
-	else
-		snprintf(buf, buf_len, "%-.*s%s", len - 3, message, TRUNC_TAIL);
+	if ((int) strlen(message) <= len)
+		tmp = my_strdup(message);
+	else {
+		tmp = my_malloc(len + 1);
+		snprintf(tmp, len + 1, "%-.*s%s", len - 3, message, TRUNC_TAIL);
+	}
 
-	return buf;
+	return tmp;
 }
 
 /*
@@ -812,8 +811,6 @@ strunc(
 wchar_t *
 wstrunc(
 	const wchar_t *wmessage,
-	wchar_t *wbuf,
-	size_t wbuf_len,
 	int len)
 {
 	wchar_t *wtmp;
@@ -822,38 +819,31 @@ wstrunc(
 	wtmp = my_wcsdup(wmessage);
 	wconvert_to_printable(wtmp);
 
-	if (wcswidth(wtmp, wcslen(wtmp)) <= len && wcslen(wtmp) < wbuf_len) {
-		/* wtmp doesn't need to be truncated */
-		wcscpy(wbuf, wtmp);
-	} else {
+	if (wcswidth(wtmp, wcslen(wtmp)) > len) {
 		/* wtmp must be truncated */
+		wchar_t *wtmp2, *tail;
+
 #	ifdef USE_UTF8_HORIZONTAL_ELLIPSIS
 		if (IS_LOCAL_CHARSET("UTF-8")) {
 			/*
 			 * use U+2026 (HORIZONTAL ELLIPSIS) instead of "..."
 			 * we gain two additional screen positions
 			 */
-			wchar_t wtail[2] = {8230, 0};	/* \0-terminated U+2026 */
-
-			wcspart(wbuf, wtmp, len - 1, wbuf_len - 1, FALSE);
-			wcscat(wbuf, wtail);
+			tail = my_malloc(sizeof(wchar_t) * 2);
+			tail[0] = 8230; /* U+2026 */
+			tail[1] = 0;	/* \0 */
 		} else
 #	endif /* USE_UTF8_HORIZONTAL_ELLIPSIS */
-		{
-			wchar_t tail[4];
-			size_t i;
+			tail = char2wchar_t(TRUNC_TAIL);
 
-			i = mbstowcs(tail, TRUNC_TAIL, ARRAY_SIZE(tail));
-			tail[3] = (wchar_t) '\0';
-			assert(i != (size_t) (-1));
-
-			wcspart(wbuf, wtmp, len - 3, wbuf_len - 3, FALSE);
-			wcscat(wbuf, tail);
-		}
+		wtmp2 = wcspart(wtmp, len - wcslen(tail), FALSE);
+		free(wtmp);
+		wtmp = my_realloc(wtmp2, sizeof(wchar_t) * (wcslen(wtmp2) + wcslen(tail) + 1));	/* wtmp2 isn't valid snymore and doesn't have to be free()ed */
+		wcscat(wtmp, tail);
+		free(tail);
 	}
-	free(wtmp);
 
-	return wbuf;
+	return wtmp;
 }
 
 
@@ -1022,14 +1012,11 @@ fmt_string(
 	const char *fmt,
 	...) {
 	char *str;
-#ifdef HAVE_VASPRINTF
-	int n;
-#endif /* HAVE_VASPRINTF */
 	va_list ap;
 
 	va_start(ap, fmt);
 #ifdef HAVE_VASPRINTF
-	if ((n = vasprintf(&str, fmt, ap)) == -1)	/* something went wrong */
+	if (vasprintf(&str, fmt, ap) == -1)	/* something went wrong */
 #endif /* HAVE_VASPRINTF */
 	{
 		size_t size = LEN;
