@@ -6,7 +6,7 @@
  *  Updated   : 2004-03-14
  *  Notes     : Split out from other modules
  *
- * Copyright (c) 1999-2004 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2005 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -85,8 +85,8 @@ look_for_multipart_info(
 	int *offset)
 {
 	MultiPartInfo tmp;
-	char *subj = (char *) 0;
-	char *pch = (char *) 0;
+	char *subj = NULL;
+	char *pch = NULL;
 
 	*offset = 0;
 
