@@ -4,7 +4,7 @@
 # signs the article and posts it.
 #
 #
-# Copyright (c) 2002-2004 Urs Janssen <urs@tin.org>,
+# Copyright (c) 2002-2005 Urs Janssen <urs@tin.org>,
 #                         Marc Brockschmidt <marc@marcbrockschmidt.de>
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,15 +33,18 @@
 #
 #
 # TODO: - FIXME add debug mode which doesn't delete tmp-files and is verbose
+#         (e.g. warns about missing mime-headers if body contains 8bit
+#          chars)
 #       - add pgp6 support
-#       - check for ~/.newsauth (and ~/.nntpauth?) and use username/password if found
+#       - check for ~/.newsauth (and ~/.nntpauth?) and use username/password
+#         if found
 #       - check for /etc/nntpserver (and /etc/news/server)
 #       - allow config in ~/.tinewsrc
 #       - add $PGPOPTS, $PGPPATH, $GNUPGHOME support
 #       - cleanup, remove duplicated code
 #
 # version Number
-my $version = "1.1.6";
+my $version = "1.1.7";
 
 my %config;
 
@@ -502,7 +505,7 @@ sub signarticle {
 	`$PGPCommand`;
 
 	open (FH, "<" . $config{'pgptmpf'} . ".txt.asc") or die "$0: can't open ".$config{'pgptmpf'}.".txt.asc: $!\n";
-	$/ = $config{'pgpbegin'}."\n";
+	$/ = "\n".$config{'pgpbegin'}."\n";
 	$_ = <FH>;
 	unless (m/\Q$config{'pgpbegin'}\E$/o) {
 		unlink $config{'pgptmpf'} . ".txt";
